function betacdf63( x, p, q, pql, iap, ier )

!-----------------------------------------------------------------------
!
!     Returns the incomplete beta function
!
!     X     - Input . Value of the variable (0 <= X <= 1)     - Real
!     P     - Input . First parameter  (P > 0)                - Real
!     Q     - Input . Second parameter (Q > 0)                - Real
!     PQL   - Input . Limit for P and Q over which the Beta   - Real
!                     distribution is approximated by a normal
!                     distribution (should be at least 1000.
!                     and should not exceed the greatest
!                     positive integer)
!     IAP   - Output. 0 if the Beta has not been approximated - Integer
!                     1 otherwise
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     2 = maximum number of iterations reached
!                         (then betacdf = value at last iteration,
!                                        or zero)
!                     3 = result out of limits
!                         (betacdf < -EPS or betacdf > 1+EPS)
!
!     External functions called:
!       BAPN (and DLGAMA if not in Fortran library)
!
!     Fortran functions called:
!       ABS  EXP  LOG  MIN  (and DLGAMA if available)
!
!     Uses continued fraction evaluation: Algorithm AS63 by
!     Majumder & Bhattacharjee; Applied Statistics, 1973, 22, 409-411
!     (when P=1 or Q=1, direct computations)
!
!     NOTE. When Beta is approximated the absolute error should not
!     be less than 0.001; otherwise it is supposed to be minimal.
!     Approximation according to:
!        D.B. Peizer, J.W. Pratt; J.A.S.A., 1968, 63, 1416-1456
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: betacdf63

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q, pql
   integer, intent(out) :: iap, ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: bapn

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: pqmin=18.0_8
   real(kind=8), parameter :: adjerr=1.0e-3_8  ! Adjusted error
   real(kind=8), parameter :: eps=0.223e-15_8, tinyr=1.0e-307_8, explower=-706.893_8
   real(kind=8), parameter :: fpmin=1.0e-300_8
   integer, parameter :: maxit=5000

   !     pqmin = lower limit for min(p,q) such that the approximation
   !             is still satisfactory (pqmin must be > 0.5). note that
   !             18 was chosen empirically, on the basis of a few trials
   !     These constants are machine dependent:
   !     fpmin = a real nearby the smallest positive real
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     tinyr = the smallest positive real
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))

   real(kind=8) :: a, aa, b, bt, c, cx, d, del, erb, h, qab, qam,  &
                   qap, rx, temp, term, xx
   integer :: m, m2
   logical :: lindx

   !--------------------------------------------------------------------

   betacdf63 = zero
   iap  = 0
   ier  = 0

   !  Test for valid input arguments

   if ( x < zero .or. x > one .or. p <= zero .or. q <= zero ) then
      ier  = 1
      betacdf63 = -one
      return
   end if

   !  x close to 0 or 1

   if ( x <= tinyr ) return
   if ( x+eps >= one ) then
      betacdf63 = one
      return
   end if

   erb = eps        !  erb is error used for final check

   !  p or q close to 1

   if ( abs(p-one) <= eps ) then
      if ( abs(q-one) <= eps ) then
         betacdf63 = x
         return
      end if
      a = q * log(one-x)
      betacdf63 = one
      if ( a >= explower ) betacdf63 = one - exp(a)
      goto 20
   end if
   if ( abs(q-one) <= eps ) then
      a = p * log(x)
      betacdf63 = zero
      if ( a >= explower ) betacdf63 = exp(a)
      goto 20
   end if

   !  p or q > pql : normal approximation (Peizer & Pratt, 1968)

   if ( q > pql .or. p > pql ) then

      iap = 1
      erb = adjerr     ! Error used in final check is adjusted

      if ( min(p,q) >= pqmin ) then
         betacdf63 = bapn( x, p, q )
         goto 20
      end if

      !  Use recurrence to reduce to the case min(p,q) >= pqmin
      if ( p <= q ) then
         xx   = x
         cx   = one - x
         a    = p
         b    = q
         term = b*log(cx) - dlgama(b)
         rx   = log(xx)
         temp = zero
         do
            bt = term + dlgama(a+b) - dlgama(a+one) + a*rx
            if ( bt >= explower ) temp = temp + exp(bt)
            a   = a + one
            if ( a >= pqmin ) exit
         end do

         if ( x <= half ) then
            !  p < q and x =< 0.5
            betacdf63 = bapn( xx, a, b ) + temp
         else
            !  p < q and x > 0.5
            betacdf63 = one - bapn( cx, b, a ) + temp
         end if

      else
         xx   = one - x
         cx   = x
         a    = q
         b    = p
         term = b*log(cx) - dlgama(b)
         rx   = log(xx)
         temp = zero
         do
            bt = term + dlgama(a+b) - dlgama(a+one) + a*rx
            if ( bt >= explower ) temp = temp + exp(bt)
            a = a + one
            if ( a >= pqmin ) exit
         end do

         if ( x < half ) then
            !  q < p and x < 0.5
            betacdf63 = one - bapn( xx, a, b ) - temp
         else
            !  q < p and x >= 0.5
            betacdf63 = bapn( cx, b, a ) - temp
         end if

      end if
      goto 20

   end if

   !  General case (p and q < pql)

   if ( x < (p+one)/(p+one+q+one) ) then
      lindx = .false.
      a  = p
      b  = q
      xx = x
   else
      lindx = .true.
      a  = q
      b  = p
      xx = one - x
   end if
   qab = a + b
   qap = a + one
   qam = a - one
   !  First step of Lentz's method
   c = one
   d = one - qab*xx/qap
   if ( abs(d) < fpmin ) d = fpmin
   d = one/d
   h = d

   do m = 1, maxit
      m2 = m+m
      aa = m*(b-m)*xx/((qam+m2)*(a+m2))
      !  One step of the recurrence (the even one)
      d = one + aa*d
      if ( abs(d) < fpmin ) d = fpmin
      c = one + aa/c
      if ( abs(c) < fpmin ) c = fpmin
      d = one/d
      h = h*d*c
      aa = -(a+m)*(qab+m)*xx/((a+m2)*(qap+m2))
      !  Next step of the recurrence (the odd one)
      d = one + aa*d
      if ( abs(d) < fpmin ) d = fpmin
      c = one + aa/c
      if ( abs(c) < fpmin ) c = fpmin
      d = one/d
      del = d*c
      h = h*del
      if ( abs(del-one) < eps ) goto 10
   end do

   !  Maximum iterations reached

   ier = 2

   10 continue
   if ( h > zero .and. a > zero ) then
      bt = dlgama(p+q) - dlgama(p) - dlgama(q) + p*log(x) + q*log(one-x)
      bt = bt + log(h) -log(a)
      if ( bt >= explower ) then
         betacdf63 = exp(bt)
      else
         betacdf63 = zero
      end if
   else
      betacdf63 = zero
   end if
   if ( lindx ) betacdf63 = one - betacdf63

   20 continue
   !  Final check, possibly adjust to 0 or 1
   if ( betacdf63 < zero ) then
      if ( betacdf63 >= -erb ) then
         betacdf63 = zero
      else
         ier = 3
      end if
   else if ( betacdf63 > one ) then
      if ( betacdf63 <= one+erb ) then
         betacdf63 = one
      else
         ier = 3
      end if
   end if

end function betacdf63

function bapn( x, p, q )

!-----------------------------------------------------------------------
!     - BAPN is called by BETACDF -
!     Normal approximation of a Beta cumulative distribution function
!     when  0 < x < 1  and  min(p,q) > 0.5
!     see Peizer & Pratt; J.A.S.A., 1968, 63, 1416-1456
!     Calls external function NCDF
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: bapn

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q

   !  Local declarations
   !  ------------------

   real(kind=8), external :: ncdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: two=one+one, three=two+one
   real(kind=8), parameter :: third=one/three, sixth=half*third
   real(kind=8), parameter :: tiny1=1.0e-2_8, tiny2=tiny1+tiny1
   real(kind=8), parameter :: eps=1.0e-15_8

   real(kind=8) :: arg, cx, d2, gp, gq, psq, uma, xn, zx

   !--------------------------------------------------------------------

   cx  = one - x
   psq = p + q
   xn  = psq - one
   d2  = (q - third + tiny2/q + (tiny1/psq))*x - (p - third + tiny2/p + (tiny1/psq))*cx
   arg = (p-half)/(xn*x)
   uma = one - arg
   if ( abs(uma) > eps ) then
      gp = ( one - arg*arg + two*arg*log(arg) ) / (uma*uma)
   else
      gp = zero
   end if
   arg = (q-half)/(xn*cx)
   uma = one - arg
   if ( abs(uma) > eps ) then
      gq = ( one - arg*arg + two*arg*log(arg) ) / (uma*uma)
   else
      gq = zero
   end if
   zx  = d2*sqrt( (one+ x*gq + cx*gp)/((xn+sixth)*cx*x) )

   !  Compute Proba(U<=zx) where U follows a normal(0,1) distribution
   bapn = ncdf( zx )

end function bapn
