function betanccdf( x, p, q, a2, delta, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Calculates the probability that a random variable distributed
!     according to the non-central beta distribution with P and Q
!     degrees of freedom and A2 eccentricity parameter, is less than
!     or equal to X
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . First degrees of freedom       (P >  0) - Real
!     Q     - Input . Second   "    "     "          (Q >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     betanccdf (stopping criteria)
!                     (eps < delta < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then betanccdf = zero)
!                     2 = maximum number of iterations reached
!                         (then betanccdf = value at last iteration,
!                                        or zero)
!                     3 = required accuracy cannot be reached
!                         (then betanccdf = value at last iteration)
!                     4 = error in auxiliary function
!                         (betacdf or phi2cdf)
!                     6 = P exceeds limit (betanccdf = zero)
!
!     External functions called:
!       BETACDF PHI2CDF
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  LOG  MIN  INT  (and DLGAMA if available)
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: betanccdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q, a2, delta
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: betacdf, phi2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: bel=1.0e6_8
   real(kind=8), parameter :: eps=0.223e-15_8, tinyr=1.0e-307_8, explower=-706.893_8
   real(kind=8), parameter :: relerr=1.0e-14_8 ! Relative error assumed
                                               ! in recurrence calculations
   real(kind=8), parameter :: xp=tinyr/relerr

   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     tinyr = the smallest positive real
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))

   real(kind=8) :: a22, a22l, dj, dj1, erp, err, gl,    &
                   kgj, kgl, kx,                        &
                   qxql, rl,                            &
                   sum, sumg,                           &
                   xl
   integer :: j, jjj

   !--------------------------------------------------------------------

   betanccdf = zero
   ier    = 0

   !  Test for valid input arguments

   if ( x < zero .or. x > one                       &
      .or. p <= zero .or. q <= zero .or. a2 < zero  &
      .or. delta >= one .or. delta <= eps ) then
      ier = 1
      return
   end if

   !  x close to 0 or 1

   if ( x <= eps ) return
   if ( x+eps >= one ) then
      betanccdf = one
      return
   end if

   !  Case a2 = 0

   if ( a2 < eps ) then
      betanccdf = betacdf( x, p, q, ier )
      if ( ier /= 0 ) ier = 4
      return
   end if

   !  If q is large enough use limiting distribution

   if ( q > bel ) then
      dj = p * p
      dj = q*x / ((dj+dj)*(one-x))
      betanccdf = phi2cdf( dj, p+p, a2, delta, maxitr, ier )
      if ( ier /= 0 ) ier = 4
      return
   end if

   !  If p is large that's all

   if ( p > bel ) then
      ier = 6
      return
   end if

   !  General case (iterations)
   !  kx's are decreasing for all j's
   !  gl's are decreasing for j >= jjj = a2/2 - 1
   !  kgj, sumg are only used for stopping rule
   !  Logs are used to avoid underflows

   err  = delta * half
   erp  = err / relerr
   a22  = a2 * half
   a22l = log( a22 )
   qxql = q * log(one-x) - dlgama(q)
   xl   = log( x )
   jjj  = int( a22 )

   !  Initialize

   gl = -a22
   kx = betacdf( x, p, q, ier )
   if ( ier /= 0 ) then
      ier = 4
      return
   end if
   kgj   = zero
   sum   = zero
   sumg  = zero
   dj    = zero

   !  Iteration loop

   do j = 0, maxitr

      if ( kx > zero ) then
         kgl = log(kx) + gl
         if ( kgl >= explower ) then
            kgl = exp( kgl )
            sum = sum + kgl
            kgj = kgj + kgl*dj
            !  Check loss of accuracy
            if ( kgj >= erp ) then
               ier = 3
               betanccdf = sum
               return
            end if
         end if
      end if

      betanccdf = sum

      !  Check accuracy (stopping rule)
      !  xp is used to prevent possible underflow

      if ( gl >= explower ) sumg = sumg + exp(gl)
      if ( kgj >= xp ) then
         if ( relerr*kgj+kx*(one-sumg) < err ) return
      else
         if (            kx*(one-sumg) < err ) return
      end if

      !  Prepare next iteration

      dj1 = j + 1
      gl  = gl + a22l - log(dj1)
      rl  = qxql + (p+dj)*xl + dlgama(p+dj+q) - dlgama(p+dj1)
      if ( rl >= explower ) kx = kx - exp( rl )
      dj = dj1

   end do

   !  Maximum number of iterations is reached

   ier = 2

end function betanccdf
