function chi2nccdf( x, p, a2, delta, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Calculates the probability that a random variable distributed
!     according to the noncentral chi-square distribution with P degrees of
!     freedom and A2 eccentricity parameter, is less than or equal to X
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . Degrees of freedom             (P >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     chi2nccdf (stopping criterion)
!                     (eps < DELTA < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then chi2nccdf = zero)
!                     2 = maximum number of iterations reached
!                         (then chi2nccdf = value at last iteration,
!                                      or zero)
!                     3 = required accuracy cannot be reached
!                         (then chi2nccdf = value at last iteration)
!                     4 = error in auxiliary function (CHI2CDF)
!
!     External functions called:
!       CHI2CDF  NORMINV
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  LOG  INT  (and DLGAMA if available)
!
!     Starting index for iterations:
!     see Frick, H. (1990), Applied Statistics, 39, 311-312.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: chi2nccdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, a2, delta
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: chi2cdf, norminv

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: dflimit=1.0e6_8, errf0=1.0e-14_8, errfa=1.0e-6_8
   real(kind=8), parameter :: eps=0.223e-15_8, tinyr=1.0e-307_8, explower=-706.893_8
   real(kind=8), parameter :: relerr=1.0e-14_8 ! Relative error assumed
                                               ! in recurrence calculations
   real(kind=8), parameter :: xp=tinyr/relerr

   !     errf0 = relative error in chi2cdf function
   !     errfa = absolute error in chi2cdf function when approximation is used
   !     dflimit = limit for approximation in chi2cdf function
   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     tinyr = the smallest positive real
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))

   real(kind=8) :: a22, a22l, dj, erp, err, err1, err2, gl,  &
                   kgj, kgl, kx,                             &
                   px2, px2l,                                &
                   p2, rl,                                   &
                   sum, sumg, xxx
   integer :: j, jjj, m

   !-----------------------------------------------------------------

   chi2nccdf = zero
   ier  = 0

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero .or. p <= zero     &
      .or. delta >= one .or. delta <= eps ) then
      ier = 1
      return
   end if

   !  Case x = 0

   if ( x < eps ) return

   !  Case a2 = 0

   if ( a2 < eps ) then
      chi2nccdf = chi2cdf( x, p, dflimit, ier )
      if ( ier /= 0 ) then
         ier = 4
      else
         if (  p <= dflimit ) then
            if ( chi2nccdf*errf0 > delta ) ier = 3
         else
            if ( errfa > delta ) ier = 3
         end if
      end if
      return
   end if

   !  General case (iterations)
   !  kx's are decreasing for all j's
   !  gl's are decreasing for j > jjj = a2/2
   !  kgj, sumg are only used for stopping rule
   !  Logs are used to avoid underflows

   err1 = delta * half
   err2 = delta - err1
   p2   = p * half
   px2  = half * x
   px2l = log( px2 )
   a22  = a2 * half
   a22l = log( a22 )
   jjj  = int( a22 )
   xxx  = norminv( err1 )
   m    = int( max(a22+xxx*sqrt(a22), zero) )

   !  Initialize

   gl = m*log(a22)-(a22+dlgama(m+one))
   kx = chi2cdf( x, p+m+m, dflimit, ier )
   if ( ier /= 0 ) then
      ier = 4
      return
   end if
   !  Take account of initial error on function
   if ( p+m+m <= dflimit ) then
      err = err2 - errf0*kx
   else
      err = err2 - errfa
   end if
   if ( err <= zero ) then
      if ( gl >= explower ) chi2nccdf = exp(gl)*kx
      ier = 3
      return
   end if
   erp  = err / relerr
   rl   = (p2+m)*px2l - ( px2 + dlgama(p2+m+one) )
   kgj  = zero
   sum  = zero
   sumg = zero
   dj   = m

   !  Iteration loop

   do j = m+0, m+maxitr

      if ( kx > zero ) then
         kgl = log(kx) + gl
         if ( kgl >= explower ) then
            kgl = exp( kgl )
            sum = sum + kgl
            kgj = kgj + kgl*dj
            !  check loss of accuracy
            if ( kgj >= erp ) then
               ier = 3
               chi2nccdf = sum
               return
            end if
         end if
      end if

      chi2nccdf = sum

      !  Check accuracy (stopping rule)
      !  xp is used to prevent possible underflow

      if ( gl >= explower ) sumg = sumg + exp(gl)
      if ( kgj >= xp ) then
         if ( relerr*kgj+kx*(one-sumg) < err ) return
      else
         if (            kx*(one-sumg) < err ) return
      end if

      !  Prepare next iteration

      dj = j + 1
      gl = gl + a22l - log(dj)
      if ( rl >= explower ) kx = kx - exp( rl )
      rl = rl - log(p2+dj) + px2l

   end do

   !  Maximum number of iterations is reached

   ier = 2

end function chi2nccdf
