function ding( x, p, a2, delta, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Calculates the probability that a random variable distributed
!     according to the noncentral chi-square distribution with P degrees of
!     freedom and A2 eccentricity parameter, is less than or equal to X
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . Degrees of freedom             (P >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     CHI2NC (stopping criterion)
!                     (eps < DELTA < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then CHI2NC = zero)
!                     2 = maximum number of iterations reached
!                         (then CHI2NC = value at last iteration,
!                                      or zero)
!                     4 = error in auxiliary function (CHI2CDF)
!
!     Subroutines called:
!       NDTRI1
!
!     External functions called:
!       CHI2CDF
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  LOG  INT  (and DLGAMA if available)
!
!     Basic algorithm:
!     Ding, C.G. (1992), Applied Statistics, 41, 478-482.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ding

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, a2 , delta
   integer, intent(out) :: maxitr, ier

   !  local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: chi2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: twol=0.6931471805599453_8      ! =log(2)
   real(kind=8), parameter :: eps=0.223e-15_8, explower=-706.893_8

   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     explower = minimum valid argument for the exponential function

   real(kind=8) :: a22, d, eps1, eps2, g, p2, sum, sumg, w
   real :: xxx
   integer :: j, m

   !-----------------------------------------------------------------

   ding = zero
   ier  = 0

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero .or. p <= zero     &
      .or. delta >= one .or. delta <= eps ) then
      ier = 1
      return
   end if

   !  Case x = 0

   if ( x < eps ) return

   !  Case a2 = 0

   if ( a2 < eps ) then
      ding = chi2cdf( x, p, 1.e4_8, ier )
      if ( ier /= 0 ) ier = 4
      return
   end if

   !  General case (iterations)
   !  m is starting index for iterations (see Frick, 1990)
   !  eps1 is the error bound for the lower truncation error
   !  eps2 is the error bound for the upper truncation error

   eps1 = delta * half
   eps2 = delta - eps1
   call ndtri1( real(eps1), xxx, ier )
   a22 = a2*half
   m = int( max(a22+xxx*sqrt(a22), zero) )
   p2 = p * half

   !  Initialize

   w = chi2cdf( x, p+m+m, 1.e4_8, ier )
   if ( ier /= 0 ) then
      ier = 4
      return
   end if
   if ( w <= zero ) return               ! since w's are decreasing
   g = m*log(a22)-(a22+dlgama(m+one))
   sum = g + log(w)
   if ( g >= explower ) then
      g = exp(g)
   else
      if ( m >= int(a22) ) return        ! since g's are decreasing
      g = zero
   end if
   sumg = g
   if ( sum >= explower ) then
      sum = exp(sum)
   else
      sum = zero
   end if
   d = (p2+m-one)*log(x) - ( x*half + (p2+m)*twol + dlgama(p2+m) )
   if ( d >= explower ) then
      d = exp(d)
   else
      d = zero
   end if

   !  Iteration loop

   do j = m+1, m+maxitr

      g = g*(a22/j)
      d = d*(x/(j+j-2+p))
      w = w-(d+d)
      !  Check accuracy (stopping rule)
      if ( (one-sumg)*w < eps2 ) then
         ding = sum
         return
      end if
      sum = sum + g*w
      sumg = sumg + g

   end do

   !  Maximum number of iterations is reached

   ier = 2
   ding = sum

end function ding
