function k2cdf( x, p, q, r, a2, delta, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Calculates the probability that a random variable distributed
!     according to the K-square distribution with P, Q and R
!     degrees of freedom and A2 eccentricity parameter, is less than
!     or equal to X
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . First degrees of freedom       (P >  0) - Real
!     Q     - Input . Second   "    "     "          (Q >  0) - Real
!     R     - Input . Third    "    "     "          (R >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     k2cdf (stopping criteria)
!                     (eps < delta < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                    -1 = no more evolution of the sum but
!                         required accuracy not reached yet
!                         (then k2cdf = value at last iteration)
!                     1 = invalid input argument
!                         (then k2cdf = zero)
!                     2 = maximum number of iterations reached
!                         (then k2cdf = value at last iteration,
!                         or zero)
!                     3 = cannot be computed
!                         (then k2cdf = zero)
!                     4 = error in auxiliary function
!                         (betacdf or lambda2cdf)
!
!     External functions called:
!       BETACDF LAMBDA2CDF
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  INT  LOG  MAX  MIN  (and DLGAMA if available)
!
!     Uses "method 2", see Benton & Krishnamoorthy (2003),
!     Computational Statistics & Data Analysis, 43, 249-267.
!     N.B. The mode is taken as the starting point for iterations
!     (forward and backward).
!     Starting index is modified if worthwile (see parameter betaratio)
!     To deal with the case where p+r<=2, xgamf is updated at the
!     end of the iteration loop rather than at the beginning.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: k2cdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q, r, a2, delta
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: betacdf, lambda2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: rlimit=1.0e6_8
   real(kind=8), parameter :: eps=0.223e-15_8, explower=-706.893_8
   real(kind=8), parameter :: betaratio=0.01_8
   integer, parameter :: kmin=10    ! When starting point of iterations is below
                                    ! this limit "method 2" is not judged worthwile

   !     These constants are machine dependent:
   !     eps = machine epsilon
   !           (the smallest real such that 1.0 + eps > 1.0)
   !     explower = minimum valid argument for the exponential function

   real(kind=8) :: a, aqa, b, beta0, betab, betaf, betak, cdf
   real(kind=8) :: gcoefb, gcoeff, prv, p2, q2, sumg, xarg, xgamb, xgamf
   integer :: j, k, kit

   !--------------------------------------------------------------------

   k2cdf = zero
   ier = 0

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero                       &
      .or. p <= zero .or. q <= zero .or. r <= zero    &
      .or. delta >= one .or. delta <= eps ) then
      ier = 1
      return
   end if

   !  Case x = 0

   if ( x < eps ) return

   xarg = p*x / (p*x+r)

   !  Case p*x/(p*x+r) = 1

   if ( abs(xarg-one) < eps ) then
      k2cdf = one
      return
   end if

   p2 = p * half
   q2 = q * half
   b = r * half

   !  If r is large enough use limiting distribution

   if ( r > rlimit ) then
      k2cdf = lambda2cdf( x, p, q, a2, delta, maxitr, ier )
      if ( ier /= 0 ) ier = 4
      return
   end if

   beta0 = betacdf( xarg, p2, b, ier ) ! Beta at step 0
   if ( ier /= 0 ) then
      ier = 4
      return
   end if

   !  Case a2 = 0

   if ( a2 < eps ) then
      k2cdf = beta0
      if ( ier /= 0 ) ier = 4
      return
   end if

   !  General case (iterations)

   aqa = a2/(q+a2)
   k = int( (q2-one)*a2/q )   ! Mode of neg. bin. distribution
   if ( k < kmin ) k = 0      ! k is not large enough to be worthwile
   a = p2 + k
   betak = betacdf( xarg, a, b, ier )
   if ( ier /= 0 ) then
      ier = 4
      return
   end if
   if ( k > 0 .and. betak < beta0*betaratio ) then ! Beta at k is small, so k is moved to
      k = xarg*k                                   ! somewhere between modes of g_j's and H_j's
      a = p2 + k
      if ( k > 0 ) then
         betak = betacdf( xarg, a, b, ier )
         if ( ier /= 0 ) then
            ier = 4
            return
         end if
      else
         betak = beta0
      end if
   end if
   xgamf = a*log(xarg) + b*log(one-xarg) + dlgama(a+b) - dlgama(a+one) - dlgama(b)
   if ( xgamf >= explower ) then
      xgamf = exp(xgamf)
   else if ( k > 0 ) then   ! xgamf at k too small for recurrence to hold
      k = 0                 ! thus start at k=0
      a = p2
      xgamf = a*log(xarg) + b*log(one-xarg) + dlgama(a+b) - dlgama(a+one) - dlgama(b)
      if ( xgamf >= explower ) then
         xgamf = exp(xgamf)
      else                           ! Cannot be computed
         ier = 3
         return
      end if
      betak = beta0
   else                              ! Cannot be computed
      ier = 3
      return
   end if
   betaf = betak
   betab = betak
   xgamb = xgamf
   gcoeff = dlgama(q2+k) - dlgama(k+one) - dlgama(q2) + k*log(aqa) + q2*log(one-aqa)
   if ( gcoeff >= explower ) then
      gcoeff = exp(gcoeff)
   else
      gcoeff = zero
   end if
   gcoefb = gcoeff
   cdf = gcoeff*betaf
   sumg = gcoeff
   prv = -one

   !  Iteration loops

   !  Do forward and backward computations k times or until convergence
   kit = min( k, maxitr )
   do j = 1, kit
         !  Forward computations
      betaf = max( betaf-xgamf, zero )
      gcoeff = gcoeff*(k+j-1+q2)*aqa/(k+j)
         !  Backward computations
      xgamb = xgamb*(1-j+a)/(xarg*(a+b-j))
      betab = betab + xgamb
      gcoefb = gcoefb*(k-j+1)/(aqa*(k-j+q2))

      cdf = cdf + gcoeff*betaf + gcoefb*betab

      sumg = sumg + gcoeff + gcoefb
      if ( (one-sumg)*beta0 <= delta ) then
         k2cdf = cdf
         return
      end if
      !  Looking for no more evolution of the sum
      if ( cdf == prv ) then
         ier = -1
         k2cdf = cdf
         return
      end if
      prv = cdf
      xgamf = xgamf*(j-1+a+b)*xarg/(j+a)
   end do

   !  Do forward computations until convergence
   do j = kit+1, maxitr
      betaf = max( betaf-xgamf, zero )
      gcoeff = gcoeff*(k+j-1+q2)*aqa/(k+j)

      cdf = cdf + gcoeff*betaf

      sumg = sumg + gcoeff
      if ( (one-sumg)*betaf <= delta ) then
         k2cdf = cdf
         return
      end if
      !  Looking for no more evolution of the sum
      if ( cdf == prv ) then
         ier = -1
         k2cdf = cdf
         return
      end if
      prv = cdf
      xgamf = xgamf*(j-1+a+b)*xarg/(j+a)
   end do

   !  Maximum number of iterations is reached

   ier = 2
   k2cdf = cdf

end function k2cdf
