function ksi2cdf( x, p, q, a2, delta, maxitr, iap, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the ksi-square distribution with P and Q degrees
!     of freedom and A2 eccentricity parameter, is less than or
!     equal to X.
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . First degrees of freedom       (P >  0) - Real
!     Q     - Input . Second   "    "     "          (Q >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on      - Real
!                     ksi2cdf (stopping criterion)
!                     (eps < DELTA < 1 where eps is machine
!                     epsilon; see parameter statement below)
!     MAXITR- Input . Maximum number of iterations            - Integer
!                     (2000 is sufficient for most cases)
!     IAP   - Output. 0 if no approximation used              - Integer
!                     1 otherwise
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then KSi2cdf = zero)
!                     2 = maximum number of iterations reached
!                         (then KSi2cdf = value at last iteration,
!                                      or zero)
!                     3 = required accuracy cannot be reached
!                         (then KSi2cdf = value at last iteration)
!                     4 = error in auxiliary function
!                         (betacdf or phi2cdf)
!
!     External functions called:
!       BETACDF  PHI2CDF
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  INT  LOG  MAX  NINT  SIGN  SQRT
!       (and DLGAMA if available)
!
!     Algorithm by J-L. Guigues (1981), improved by B. Lecoutre (1986)
!
!     J. Poitevineau (CNRS-URA1201)
!     January 01, 1991 (First version 26/04/85)
!     Modif.  23/09/93 (jjj)  25/11/93 (jj, zg)  10/08/94 (y1, phi2cdf)
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ksi2cdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q, a2, delta
   integer, intent(in) :: maxitr
   integer, intent(out) :: iap, ier

   !  local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: betacdf, phi2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: two=one+one
   real(kind=8), parameter :: qlimit=1.0e9_8
   real(kind=8), parameter :: eps=0.223e-15_8, tinyr=1.0e-307_8, explower=-706.893_8
   real(kind=8), parameter :: relerr=1.0e-14_8 ! Relative error assumed
                                               ! in recurrence calculations
   real(kind=8), parameter :: xp=tinyr/relerr
   real(kind=8), parameter :: xmxint=2147483647.0_8

   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     tinyr = the smallest positive real
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))
   !     xmxint= maximum positive integer

   real(kind=8) :: aqal, bl, cl, dax, dj, dj1,  &
                   erp, err, g, gcj, gcl,       &
                   pq2, p2, qqal, q2, r, rl,    &
                   sum, sumc,                   &
                   v, w, y, yyl,                &
                   y1, z, zg, zm, zn
   integer :: iok, j, jj, jok
   logical :: lalf

   !-----------------------------------------------------------------

   ksi2cdf = zero
   iap = 0
   ier = 0

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero .or. p <= zero .or. q <= zero  &
      .or. delta >= one .or. delta <= eps ) then
      ier  = 1
      return
   end if

   y  = x      / (a2+q+x)
   y1 = (a2+q) / (a2+q+x)

   !  y near 0 or 1

   if ( y  <= tinyr ) return
   if ( y1 <= tinyr ) then
      ksi2cdf = one
      return
   end if

   !  Define usefull parameters

   p2  = p * half
   q2  = q * half
   pq2 = p2 + q2

   !  Case a2 = 0

   if ( a2 < eps ) then
      ksi2cdf = betacdf( x/(x+q), p2, q2, iok )
      if ( iok /= 0 ) ier = 4
      return
   end if

   !  Case p = 1

   if ( abs(p-one) < eps ) then
      dax  = two * sqrt( a2*x )
      ksi2cdf = half * ( sign( one, x-a2 ) *                     &
                betacdf( (a2+x-dax)/(a2+x-dax+q),half,q2,iok ) + &
                betacdf( (a2+x+dax)/(a2+x+dax+q),half,q2,jok ) )
      if ( iok /= 0 .or. jok /= 0 ) ier = 4
      return
   end if

   ! If q is large enough use limiting distribution

   if ( q > qlimit ) then
      ksi2cdf = phi2cdf( x/p, p, a2, delta, maxitr, ier )
      if ( ier /= 0 ) ier = 4
      iap = 1
      return
   end if

   !  Calculate 1-f(x) or f(x) (lalf is the indicator)

   lalf = .false.
   z    = y
   zm   = p2
   zn   = q2
   if ( abs(y-half) < eps ) then
      !  Look for special case y = 0.5 and p = q
      if ( abs(p-q) < eps ) then
         ksi2cdf = half
         return
      end if
      if ( p > q ) then
         lalf = .true.
         zm   = q2
         zn   = p2
      end if
   else if ( y > half ) then
      lalf = .true.
      z    = y1
      zm   = q2
      zn   = p2
   end if

   !  General case (iterations)
   !  g's  are decreasing for j >= jj = -v/w + 1
   !  cl's are decreasing for j >= jjj = a2/2 - a2/q - 1
   !  gcj, sumc, zg are only used for stopping rule
   !  Logs are used to avoid underflows

   !  Define constants

   err  = delta * half
   erp  = err / relerr
   yyl  = log( y*y1 )
   qqal = q2 * log( q/(q+a2) )
   aqal = log( a2/(q+a2) )
   v    = pq2*z - zn
   w    = z + z - one

   !  Initialize

   bl = zero
   cl = qqal
   g  = betacdf( z, zm, zn, iok )
   if ( iok /= 0 ) then
      ier  = 4
      return
   end if
   zg = g
   rl = p2*log(y) + q2*log(y1) + dlgama(pq2) - dlgama(p2) - dlgama(q2) - log(p2) - log(q2)
   if ( rl >= explower ) then
      r = exp( rl )
   else
      r = zero
   end if
   sum  = zero
   gcj  = zero
   sumc = zero

   !  Define jj (and associated upper bound zg)
   !  (note that w=0 if and only if y=0.5, and in such a case zm<zn)

   if ( zm > zn ) then
      zg = -v/w
      if ( abs(zg) < xmxint ) then
         jj = int( zg ) + 1
         if ( jj > 0 ) then
            zg = betacdf( z, zm+jj, zn+jj, iok )
            if ( iok /= 0 ) then
               ier  = 4
               return
            end if
         end if
      else
         zg = one
         jj = maxitr + 1
      end if
   else
      jj = 0
   end if

   dj = zero

   !  Iteration loop

   do j = 0, maxitr

      if ( g > zero ) then
         gcl = log(g) + cl
         if ( gcl >= explower ) then
            gcl = exp( gcl )
            sum = sum + gcl
            gcj = gcj + gcl*dj
            !  Check loss of accuracy
            if ( gcj >= erp ) then
               ier = 3
               goto 10
            end if
         end if
      end if

      !  Check accuracy (stopping rule)
      !  xp is used to prevent possible underflow

      if ( cl >= explower ) sumc = sumc + exp(cl)
      if ( j >= jj ) zg = g
      if ( gcj >= xp ) then
         if ( relerr*gcj+zg*(one-sumc) < err ) goto 10
      else
         if (            zg*(one-sumc) < err ) goto 10
      end if

      !  Prepare next iteration

      dj1 = j + 1
      bl  = bl + log( (q2+dj)/dj1 )
      cl  = bl + qqal + dj1*aqal
      g   = g  + (v+w*dj)*r
      rl  = rl + yyl + log( ((dj+dj+pq2)/(dj1+p2)) * ((dj+dj1+pq2)/(dj1+q2)) )
      if ( rl >= explower ) then
         r = exp( rl )
      else
         r = zero
      end if
      dj = dj1

   end do

   !  Maximum number of iterations is reached

   ier = 2

   !  The end

   10 continue
   if ( lalf ) then
      ksi2cdf = one - sum
   else
      ksi2cdf = sum
   end if

end function ksi2cdf
