function binomccdf( x, n, p, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the continued binomial distribution with sample size
!     N and parameter P, is less than or equal to X. The function is
!     defined on the interval (-1; N).
!
!     X     - Input . Value of the variable (-1 <= X <= N)    - Real
!     N     - Input . Sample size (N > 0)                     - Integer
!     P     - Input . parameter of the binomial (0 <= P <= 1) - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     2 = maximum number of iterations reached
!                         (then binomccdf = value at last iteration,
!                                        or zero)
!                     3 = result out of limits
!                         (binomccdf < 0 or binomccdf > 1)
!
!     External functions called:
!       BINOMCDF  BINOMPMF
!
!     Fortran functions called:
!       INT
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: binomccdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p
   integer, intent(in) :: n
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: binomcdf, binompmf

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8) :: t
   integer :: m

   !--------------------------------------------------------------------

   binomccdf = zero
   ier = 0

   !  Test for valid input arguments

   if ( n <= 0 .or. x < -one .or. x > n .or. p < zero .or. p > one ) then
      ier = 1
      binomccdf = -one
      return
   end if

   !  Special case

   if ( x >= n ) then
      binomccdf = one
      return
   end if
   if ( p == one ) return

   !  General case

   if ( x < zero ) then
      m = -1
      t = zero
   else
      m = int(x)
      t = binomcdf( m, n, p, ier )
      if ( ier /= 0 ) return
   end if
   binomccdf = t + (x-m)*binompmf( m+1, n, p, ier )

end function binomccdf
