function chgm( x, p, q, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the confluent hypergeometric distribution with
!     parameters P and Q, is less than or equal to X.
!
!     X     - Input . Value of the variable                   - Real
!     P     - Input . 1st (numerator) parameter               - Real
!     Q     - Input . 2nd (denominator) parameter             - Real
!                     (Q /= 0,-1,-2,...)
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then CHGM = 0)
!
!     External functions called:
!       DGAMMA if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  COS  EXP  INT  (and DGAMMA if available)
!
!     From Zhang S. & Jin J. (1996). Computation of Special Functions.
!          New York: Wiley.
!
!     Warning.
!     In the case of a << -1 when x > 0 and a >> 1 when x < 0, the
!     evaluation involves the summation of a partially alternating
!     series which results in a loss of significant digits.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: chgm

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dgamma

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8, two=2.0_8
   real(kind=8), parameter :: pi=3.141592653589793_8
   real(kind=8), parameter :: eps=0.223e-15_8
                            ! eps = machine epsilon
                            ! (smallest real such that 1.0+eps > 1.0)
   real(kind=8) :: a, a0, a1, b, hg, hg1, hg2, r, rg, r1, r2, sum1, sum2, ta, tb, tba, y0, y1, z
   integer :: i, j, k, l, l0, m, nl

   !--------------------------------------------------------------------

   ier = 0
   hg = zero
   z = x
   a = p
   b = q
   a0 = a
   a1 = a

   if ( b == zero .or. b == -abs(int(b)) ) then   ! Invalid parameter
      chgm = zero
      ier = 1
      return
   else if ( a == zero .or. z == zero ) then
      hg = one
   else if ( a == -one ) then
      hg = one-z/b
   else if ( a == b ) then
      hg = exp(z)
   else if ( a-b == one ) then
      hg = (one+z/b)*exp(z)
   else if ( a == one .and. b == two ) then
      hg = (exp(z)-one)/z
   else if ( a < zero .and. a == int(a) ) then
      m = int(-a)
      r = one
      hg = one
      do k = 1, m
         r = r*(k-1+a)/k/(k-1+b)*z
         hg = hg + r
      end do
   end if
   chgm = hg
   if ( hg /= zero ) return

   if ( z < zero ) then
      a = b-a
      a0 = a
      z = abs(z)
   end if
   if ( a < two ) then
      nl = 0
   else
      nl = 1
      l = int(a)
      a = a-(l+1)
   end if
   do l0 = 0, nl
      if ( a0 >= two ) a = a+one
      if ( a < zero .or. z <= 30.0_8+abs(b) ) then
         hg = one
         rg = one
         do j = 1, 500
            rg = rg*(j-1+a)/(j*(j-1+b))*z
            hg = hg + rg
            if ( abs(rg/hg) < eps ) goto 10
         end do
      else
         ta = dgamma(a)
         tb = dgamma(b)
         tba = dgamma(b-a)
         sum1 = one
         sum2 = one
         r1 = one
         r2 = one
         do i = 1, 8
            r1 = -r1*(i-1+a)*(a-b+i)/(z*i)
            r2 = -r2*(i-1+b-a)*(a-i)/(z*i)
            sum1 = sum1 + r1
            sum2 = sum2 + r2
         end do
         hg1 = tb/tba*z**(-a)*cos(pi*a)*sum1
         hg2 = tb/ta*exp(z)*z**(a-b)*sum2
         hg = hg1 + hg2
      end if
      10 continue
      if ( l0 == 0 ) y0 = hg
      if ( l0 == 1 ) y1 = hg
   end do
   if ( a0 >= two ) then
      do i = 1, l-1
         hg = ((a+a-b+z)*y1+(b-a)*y0)/a
         y0 = y1
         y1 = hg
         a = a + one
      end do
   end if
   if ( x < zero ) hg = hg*exp(x)
   chgm = hg

end function chgm
