function confhypg( x, p, q, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the confluent hypergeometric distribution with
!     parameters P and Q, is less than or equal to X.
!
!     X     - Input . Value of the variable                   - Real
!     P     - Input . 1st (numerator) parameter               - Real
!     Q     - Input . 2nd (denominator) parameter             - Real
!                     (Q /= 0,-1,-2,...)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then confhypg = -1.)
!                     2 = maximum number of iterations reached
!                         (then confhypg = value at last iteration)
!
!     External functions called:
!       NONE
!
!     Fortran functions called:
!       ABS  INT
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: confhypg

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8) :: dj, s, sum
   integer :: j

   !--------------------------------------------------------------------

   ier = 0

   !  Test for valid input arguments

   if ( q == zero .or. q == -abs(int(q)) ) then
      confhypg = -one
      ier = 1
      return
   end if

   confhypg = one

   !  Case x = 0

   if ( x == zero ) return

   !  Iteration loop

   dj = zero
   s = confhypg
   sum = confhypg
   do j = 1, maxitr
      s = s*((p+dj)/(q+dj))
      dj = j
      s = s*(x/dj)
      confhypg = confhypg + s
      if ( confhypg == sum ) return    ! end of iterations
      sum = confhypg
   end do

   !  Maximum number of iterations is reached

   ier = 2

end function confhypg
