function erfun( x )

!-----------------------------------------------------------------------
!     Returns the error function
!
!     X     - Input . Argument                                - Real
!
!-----------------------------------------------------------------------
!
!   The main computation evaluates near-minimax approximations
!   from "Rational Chebyshev approximations for the error function"
!   by W. J. Cody, Math. Comp., 1969, PP. 631-638.  This
!   transportable program uses rational functions that theoretically
!   approximate  erf(x)  and  erfc(x)  to at least 18 significant
!   decimal digits.  The accuracy achieved depends on the arithmetic
!   system, the compiler, the intrinsic functions, and proper
!   selection of the machine-dependent constants.
!
!*******************************************************************
!
! Explanation of machine-dependent constants
!
!   XMIN   = the smallest positive floating-point number.
!   XINF   = the largest positive finite floating-point number.
!   XNEG   = the largest negative argument acceptable to ERFCX;
!            the negative of the solution to the equation
!            2*exp(x*x) = XINF.
!   XSMALL = argument below which erf(x) may be represented by
!            2*x/sqrt(pi)  and above which  x*x  will not underflow.
!            A conservative value is the largest machine number X
!            such that   1.0 + X = 1.0   to machine precision.
!   XBIG   = largest argument acceptable to ERFC;  solution to
!            the equation:  W(x) * (1-0.5/x**2) = XMIN,  where
!            W(x) = exp(-x*x)/[x*sqrt(pi)].
!   XHUGE  = argument above which  1.0 - 1/(2*x*x) = 1.0  to
!            machine precision.  A conservative value is
!            1/[2*sqrt(XSMALL)]
!   XMAX   = largest acceptable argument to ERFCX; the minimum
!            of XINF and 1/[sqrt(pi)*XMIN].
!
!   Approximate values for some important machines are:
!
!                          XMIN       XINF        XNEG     XSMALL
!
!  CDC 7600      (S.P.)  3.13E-294   1.26E+322   -27.220  7.11E-15
!  CRAY-1        (S.P.)  4.58E-2467  5.45E+2465  -75.345  7.11E-15
!  IEEE (IBM/XT,
!    SUN, etc.)  (S.P.)  1.18E-38    3.40E+38     -9.382  5.96E-8
!  IEEE (IBM/XT,
!    SUN, etc.)  (D.P.)  2.23D-308   1.79D+308   -26.628  1.11D-16
!  IBM 195       (D.P.)  5.40D-79    7.23E+75    -13.190  1.39D-17
!  UNIVAC 1108   (D.P.)  2.78D-309   8.98D+307   -26.615  1.73D-18
!  VAX D-Format  (D.P.)  2.94D-39    1.70D+38     -9.345  1.39D-17
!  VAX G-Format  (D.P.)  5.56D-309   8.98D+307   -26.615  1.11D-16
!
!
!                          XBIG       XHUGE       XMAX
!
!  CDC 7600      (S.P.)  25.922      8.39E+6     1.80X+293
!  CRAY-1        (S.P.)  75.326      8.39E+6     5.45E+2465
!  IEEE (IBM/XT,
!    SUN, etc.)  (S.P.)   9.194      2.90E+3     4.79E+37
!  IEEE (IBM/XT,
!    SUN, etc.)  (D.P.)  26.543      6.71D+7     2.53D+307
!  IBM 195       (D.P.)  13.306      1.90D+8     7.23E+75
!  UNIVAC 1108   (D.P.)  26.582      5.37D+8     8.98D+307
!  VAX D-Format  (D.P.)   9.269      1.90D+8     1.70D+38
!  VAX G-Format  (D.P.)  26.569      6.71D+7     8.98D+307
!
!*******************************************************************
!
! Intrinsic functions required are:
!     ABS, AINT, EXP
!
!  Author: W. J. Cody
!          Mathematics and Computer Science Division
!          Argonne National Laboratory
!          Argonne, IL 60439
!
!  Latest modification: March 19, 1990
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: erfun

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x

   !  Local declarations
   !  ------------------

   integer :: i
   real(kind=8) :: del, result, xden, xnum, y, ysq

      !  Mathematical constants:
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8, four=4.0_8, sixten=16.0_8
   real(kind=8), parameter :: sqrpi=5.6418958354775628695e-1_8, thresh=0.46875_8

      !  Machine-dependent constants:
   real(kind=8), parameter :: xinf=1.79e308_8, xneg=-26.628_8, xsmall=1.11e-16_8
   real(kind=8), parameter :: xbig=26.543_8, xhuge=6.71e7_8, xmax=2.53e307_8

      !  Coefficients for approximation to  erf  in first interval:
   real(kind=8), dimension(5) :: a=(/                             &
               3.16112374387056560_8,   1.13864154151050156e02_8, &
               3.77485237685302021e02_8,3.20937758913846947e03_8, &
               1.85777706184603153e-1_8/)
   real(kind=8), dimension(4) :: b=(/                             &
               2.36012909523441209e01_8,2.44024637934444173e02_8, &
               1.28261652607737228e03_8,2.84423683343917062e03_8/)

      !  Coefficients for approximation to  erfc  in second interval:
   real(kind=8), dimension(9) :: c=(/                             &
               5.64188496988670089e-1_8,8.88314979438837594_8,    &
               6.61191906371416295e01_8,2.98635138197400131e02_8, &
               8.81952221241769090e02_8,1.71204761263407058e03_8, &
               2.05107837782607147e03_8,1.23033935479799725e03_8, &
               2.15311535474403846e-8_8/)
   real(kind=8), dimension(8) :: d=(/                             &
               1.57449261107098347e01_8,1.17693950891312499e02_8, &
               5.37181101862009858e02_8,1.62138957456669019e03_8, &
               3.29079923573345963e03_8,4.36261909014324716e03_8, &
               3.43936767414372164e03_8,1.23033935480374942e03_8/)

      !  Coefficients for approximation to  erfc  in third interval:
   real(kind=8), dimension(6) :: p=(/                             &
               3.05326634961232344e-1_8,3.60344899949804439e-1_8, &
               1.25781726111229246e-1_8,1.60837851487422766e-2_8, &
               6.58749161529837803e-4_8,1.63153871373020978e-2_8/)
   real(kind=8), dimension(5) :: q=(/                             &
               2.56852019228982242_8,   1.87295284992346047_8,    &
               5.27905102951428412e-1_8,6.05183413124413191e-2_8, &
               2.33520497626869185e-3_8/)

   !--------------------------------------------------------------------

   y = abs(x)

   if ( y <= thresh ) then
         !--------------------------------------!
         !  Evaluate  erf  for  |X| <= 0.46875  !
         !--------------------------------------!
      ysq = zero
      if ( y > xsmall ) ysq = y * y
      xnum = a(5)*ysq
      xden = ysq
      do i = 1, 3
         xnum = (xnum + a(i)) * ysq
         xden = (xden + b(i)) * ysq
      end do
      erfun = x * (xnum + a(4)) / (xden + b(4))
      return

   else if ( y <= four ) then
         !---------------------------------------------!
         !  Evaluate  erfc  for 0.46875 <= |X| <= 4.0  !
         !---------------------------------------------!
      xnum = c(9)*y
      xden = y
      do i = 1, 7
         xnum = (xnum + c(i)) * y
         xden = (xden + d(i)) * y
      end do
      result = (xnum + c(8)) / (xden + d(8))
      ysq = aint(y*sixten)/sixten
      del = (y-ysq)*(y+ysq)
      result = exp(-ysq*ysq) * exp(-del) * result

   else
         !---------------------------------!
         !  Evaluate  erfc  for |X| > 4.0  !
         !---------------------------------!
      result = zero
      if ( y >= xbig ) then
         erfun = (half - result) + half
         if ( x < zero ) erfun = -erfun
         return
      end if
      ysq = one / (y * y)
      xnum = p(6)*ysq
      xden = ysq
      do i = 1, 4
         xnum = (xnum + p(i)) * ysq
         xden = (xden + q(i)) * ysq
      end do
      result = ysq *(xnum + p(5)) / (xden + q(5))
      result = (sqrpi -  result) / y
      ysq = aint(y*sixten)/sixten
      del = (y-ysq)*(y+ysq)
      result = exp(-ysq*ysq) * exp(-del) * result
   end if

      !--------------------------------!
      !  Fix up for negative argument  !
      !--------------------------------!
   erfun = (half - result) + half
   if ( x < zero ) erfun = -erfun

end function erfun
