function fdnccdf( x, df1, df2, alamb1, alamb2, eps, iflag )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the doubly noncentral F distribution with DF1 and
!     DF2 degrees of freedom, ALAMB1 and ALAMB2 noncentralitry
!     parameters, is less than or equal to X.
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     DF1   - Input . First degrees of freedom       (DF1> 0) - Real
!     DF2   - Input . Second   "    "     "          (DF2> 0) - Real
!     ALAMB1- Input . Noncentrality parameter for (ALAMB1>=0) - Real
!                     the numerator
!     ALAMB2- Input . Noncentrality parameter for (ALAMB2>=0) - Real
!                     the denominator
!     EPS   - Input . Desired absolute accuracy  (EPS>=1E-10) - Real
!     IFLAG - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then fdnccdf = -1.)
!                     2 = error in auxiliary function betacdf
!                         (then fdnccdf = value at last iteration,
!                                        or zero)
!                     3 = vector dimensions too small
!                         (increase nx)
!
!     This is a simple Fortran90 adaptation of subroutine CDFDNF by
!     C.P. Reeve:
!     Reeve, C.P. (1986). An algorithm for computing the doubly
!     noncentral F c.d.f. to a specified accuracy.
!     Statistical Engineering Division, note 86-4, november.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: fdnccdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, df1, df2, alamb1, alamb2, eps
   integer, intent(out) :: iflag

   !  Local declarations
   !  ------------------

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   integer, parameter :: nx=10000

   real(kind=8) :: cdfx, fa, fb, fc, ga, gb, gc, xx, yy
   real(kind=8) :: bfi(1:nx), bfj(1:nx), poi(1:nx), poj(1:nx)
   integer :: i, imin, j, jmin, ni, nj

   !-----------------------------------------------------------------

   !  Test for valid input arguments

   if ( x < zero .or. df1 <= zero .or. df2 <= zero .or.   &
        alamb1 < zero .or. alamb2 < zero .or.             &
        eps < 1.e-10_8 .or. eps >= one ) then
      iflag = 1
      fdnccdf = -one
      return
   end if

   fdnccdf = zero
   iflag = 0

   fa = half*alamb1
   ga = half*alamb2
   fb = half*df1
   gb = half*df2
   yy = df2/(df2+df1*x)
   if ( yy >= one ) return
   xx = one - yy
   if ( xx >= one ) then
      fdnccdf = one
      return
   end if
   cdfx = zero

   !  Compute Poisson probabilities in vectors poi and poj

   call poissf( fa, eps, imin, ni, poi(:nx), nx, iflag )
   if ( iflag /= 0 ) return
   fc = fb + imin
   call poissf( ga, eps, jmin, nj, poj(:nx), nx, iflag )
   if ( iflag /= 0 ) return
   gc = gb + jmin

   !  Compute Beta cdf by recurrence when i=imin and j=jmin to jmax

   call edgef( nj, gc, fc, yy, xx, bfj(1:nj), cdfx, poj(1:nj), poi(1:1), iflag, 1 )
   if ( ni <= 1 .or. iflag /= 0 ) then
      fdnccdf = cdfx
      return
   end if

   !  Compute Beta cdf by recurrence when j=jmin and i=imin to imax

   bfi(1) = bfj(1)
   call edgef( ni, fc, gc, xx, yy, bfi(1:ni), cdfx, poi(1:ni), poj(1:1), iflag, 2 )
   if ( nj <= 1 .or. iflag /= 0 ) then
      fdnccdf = cdfx
      return
   end if

   !  Compute Beta cdf by recurrence when i>imin and j>jmin

   do i = 2, ni
      bfj(1) = bfi(i)
      do j = 2, nj
         bfj(j) = xx*bfj(j) + yy*bfj(j-1)
         cdfx = cdfx + poi(i)*poj(j)*bfj(j)
      end do
   end do

   fdnccdf = cdfx

end function fdnccdf
subroutine poissf( alamb, eps, l, nspan, v, nv, iflag )
   ! Compute the Poisson(alamb) probabilities over the range (l,k)
   ! where the total tail probability is less than eps/2, sum the
   ! probabilities and shift them to the beginning of vector v.

   implicit none

   !  Arguments
   real(kind=8) :: alamb, eps
   integer :: l, nspan, nv, iflag
   real(kind=8) :: v(1:nv)

   !  Local declarations
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8) :: dal, dk, dlimit, dsum, pk, pl
   integer :: i, inc, k, nk, nl

   dlimit = one-half*eps
   k = int(alamb)
   l = k+1
   if ( alamb == zero ) then
      pl = one
   else
      dal = alamb
      dk = k
      pl = exp(dk*log(dal)-dal-dlgama(k+one))
   end if
   pk = alamb*pl/l
   nk = nv/2
   nl = nk+1
   dsum = zero
   do
      if ( pl < pk ) then
         nk = nk+1
         if ( nk > nv ) then
            iflag = 3
            return
         end if
         v(nk) = pk
         dsum =dsum + pk
         k = k+1
         if ( dsum >= dlimit ) exit
         pk = alamb*pk/(k+1)
      else
         nl = nl-1
         v(nl) = pl
         dsum = dsum + pl
         l = l-1
         if ( dsum >= dlimit ) exit
         pl = l*pl/alamb
      endif
   end do
   inc = nl-1
   do i = nl, nk
      v(i-inc) = v(i)
   end do
   nspan = nk-inc

end subroutine poissf

subroutine edgef( nk, fc, gc, xx, yy, bfk, cdfx, poi, poj, iflag, l )
   ! Compute the Beta cdf's by a recurrence relation along the edges
   ! i=imin and j=jmin of a grid. The corresponding components of
   ! the F'' cdf are included in the summation. Terms which might
   ! cause underflow are set to zero.

   implicit none

   !  Arguments
   real(kind=8) :: fc, gc, xx, yy, cdfx
   integer :: nk, iflag, l
   real(kind=8) :: bfk(1:nk), poi(1:nk), poj(1:1)

   !  Local declarations
   real(kind=8), external :: betacdf
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: deuflo=-706.893_8
   real(kind=8) :: darg, di, dk, fd, fk
   integer :: i, j, k, kflag

   fd = fc-one
   k = max(l,min(nk,int((gc-one)*xx/yy-fd)))
   fk = fd+k
   bfk(k) = betacdf( xx,fk,gc,iflag )
   if ( iflag /= 0 ) then
      iflag = 2
      return
   end if
   if ( l == 1 ) bfk(k) = one-bfk(k)
   if ( nk /= 1 ) then
      darg = fk*log(xx)+gc*log(yy)-log(fk)+dlgama(fk+gc)-dlgama(fk)-dlgama(gc)
      if ( darg < deuflo ) then
         dk = zero
      else
         dk = exp(darg)*(-one)**l
      end if
      if ( k < nk ) then
         bfk(k+1) = bfk(k)-dk
         di = dk
         kflag = 1
         do i = k+1, nk-1
            if ( kflag == 1 ) then
               di = di*(fd+gc+(i-1))*xx/(fd+i)
               if ( dk+di == dk ) then
                  kflag = 0
                  di = zero
               end if
            end if
            bfk(i+1) = bfk(i)-di
         end do
      end if
      di = dk
      kflag = 1
      do i = k-1, l, -1
         if ( kflag == 1 ) then
            di = di*(fc+i)/((fd+gc+i)*xx)
            if ( dk+di == dk ) then
               kflag = 0
               di = zero
            end if
         end if
         bfk(i) = bfk(i+1)+di
      end do
   end if
   do i = l, nk
      cdfx = cdfx + poi(i)*poj(1)*bfk(i)
   end do

end subroutine edgef
