function hypergcdf( x, n, m, npop, e, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the hypergeometric distribution with parameters
!     N, NPOP, M is less than or equal to X.
!
!     X     - Input . Value of the variable: number of        - Integer
!                     "positive" events in the sample
!                     (0 <= X <= N)
!     N     - Input . Sample size (0 < N <= NPOP)             - Integer
!     M     - Input . Number of "positive" events in the      - Integer
!                     population (0 <= M <= NPOP)
!     NPOP  - Input . Population size (NPOP > 0)              - Integer
!     E     - Input . Accuracy (1E-14 <= E < 1)               - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!
!     External functions called:
!       INVJM
!
!     Fortran functions called:
!       MAX
!
!     Uses Berkopec's alogorithm:
!     A. Berkopec, HyperQuick alogorithm for discrete hypergeometric
!     distribution. Journal of Discrete Algorithms, 2007, 5, 341-347,
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: hypergcdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: e
   integer, intent(in) :: x, n, m, npop
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: invjm

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: emx=1.0e-14_8   ! maximum accuracy
   real(kind=8) :: ak, bk, ck, ee, epsk, jjm, s
   integer :: k

   !--------------------------------------------------------------------

   !  Test for valid input arguments

   if ( npop <= 0 .or. m < 0 .or. n <= 0 .or. n > npop .or.  &
        x < 0 .or. x > n .or. e <= zero .or. e >= one ) then
      ier = 1
      hypergcdf = -one
      return
   end if

   hypergcdf = zero
   ier = 0

   !  Special cases

   if ( m == 0 ) then
      hypergcdf = one
      return
   else if ( m == npop .and. x < m ) then
      return
   else if ( m == npop .and. x == m ) then
      hypergcdf = one
      return
   else if ( n == npop ) then
      if ( x >= m ) hypergcdf = one
      return
   end if

   !  General case

   s = one
   do k = x, m-2
      s = s*invjm(n,x,npop,k) + one
   end do
   ak = s
   bk = s
   k = m - 2
   ee = max( e, emx )
   epsk = ee + ee
   do
      if ( k >= npop-(n-x)-1 .or. epsk <= ee ) exit
      ck = ak/bk
      k = k + 1
      jjm = invjm(n,x,npop,k)
      bk = bk*jjm + one
      ak = ak*jjm
      epsk = (npop-(n-x)-1-k)*(ck-ak/bk)
   end do
   hypergcdf = one - (ak/bk-epsk*half)

end function hypergcdf

function invjm( n, x, npop, m )
   real(kind=8) :: invjm
   integer, intent(in) :: n, x, npop, m
   real(kind=8), parameter :: one=1.0_8

   invjm = (one-x/(m+one))/(one-(n-x-one)/(npop-m-one))
end function invjm
