function tdnccdf( x, df, delta, alamb, eps, iflag )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the doubly noncentral t distribution with DF
!     degrees of freedom, DELTA and ALAMB noncentralitry
!     parameters, is less than or equal to X.
!
!     X     - Input . Value of the variable                   - Real
!     DF    - Input . Degrees of freedom in the      (DF > 0) - Real
!                     denominator
!     DELTA - Input . Noncentrality parameter for             - Real
!                     the numerator
!     ALAMB - Input . Noncentrality parameter for  (ALAMB>=0) - Real
!                     the denominator
!     EPS   - Input . Desired absolute accuracy  (EPS>=1E-10) - Real
!     IFLAG - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then tdnccdf = -1.)
!                     2 = error in auxiliary function betacdf
!                         (then tdnccdf = value at last iteration,
!                                        or zero)
!                     3 = vector dimensions too small
!                         (increase nx)
!
!     This is a simple Fortran90 adaptation of subroutine CDFDNT by
!     C.P. Reeve:
!     Reeve, C.P. (1986). An algorithm for computing the doubly
!     noncentral t c.d.f. to a specified accuracy.
!     Statistical Engineering Division, note 86-5, december.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: tdnccdf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, df, delta, alamb, eps
   integer, intent(out) :: iflag

   !  Local declarations
   !  ------------------

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   integer, parameter :: nx=10000

   real(kind=8) :: cdfx, darg, delsq, fa, fc, ga, gb, gc, sdelta, sum, xx, yy
   real(kind=8) :: bfi(1:nx), bfj(1:nx), poi(1:nx), poj(1:nx)
   integer :: i, imin, jmin, k, l, ni, nj
   logical :: ll

   !-----------------------------------------------------------------

   !  Test for valid input arguments

   if ( df <= zero .or. alamb < zero .or.      &
        eps < 1.e-10_8 .or. eps >= one ) then
      iflag = 1
      tdnccdf = -one
      return
   end if

   cdfx = zero
   tdnccdf = zero
   iflag = 0

   delsq = delta**2
   fa = half*delsq
   ga = half*alamb
   gb = half*df
   yy = df/(df+x*x)
   xx = one - yy

   !  If x<0 set ll=true, reverse sign of delta and use the
   !  identity described up front for computing the cdf

   ll = x < zero
   if ( xx >= one ) then
      tdnccdf = one
      if ( ll ) tdnccdf = zero
      return
   end if
   sdelta = delta
   if ( ll ) sdelta = -sdelta

   !  Compute Poisson probabilities in vector poi

   call poisst( fa, eps, imin, ni, poi(:nx), nx, iflag )
   if ( iflag /= 0 ) return
   if ( yy < one ) then
      fc = half + imin
      !  Compute Poisson probabilities in vector poj
      call poisst( ga, eps, jmin, nj, poj(:nx), nx, iflag )
      if ( iflag /= 0 ) return
      gc = gb + jmin

      !  Sum the terms corresponding to even values of index i

      call grid( ni, nj, fc, gc, bfi(1:ni), bfj(1:nj), poi(1:ni), poj(1:nj), xx, yy, cdfx, iflag )
      if ( iflag /= 0 ) then
         tdnccdf = cdfx
         return
      end if
   end if
   if ( delta == zero ) then
      ni = 0
      sum = zero
      if ( yy >= one ) then
         cdfx = half*(cdfx+one-sum)
         if ( ll ) cdfx = one - cdfx
         tdnccdf = cdfx
         return
      end if
   else

      !  Compute Poisson-like probabilities in vector poi

      k = int(fa)
      if ( imin > 0 ) then
         imin = imin-1
         ni = ni+1
      end if
      darg = (k+half)*log(fa)-fa-dlgama(k+one+half)
      l = k-imin+1
      poi(l) = sign(exp(darg),sdelta)
      sum = poi(l)
      do i = k-1, imin, -1
         l = l-1
         poi(l) = poi(l+1)*(i+one+half)/fa
         sum = sum+poi(l)
      end do
      l = k-imin+1
      do i = k+1, imin+ni-1
         l = l+1
         poi(l) = poi(l-1)*fa/(i+half)
         sum = sum+poi(l)
      end do
      if ( yy >= one ) then
         cdfx = half*(cdfx+one-sum)
         if ( ll ) cdfx = one - cdfx
         tdnccdf = cdfx
         return
      end if
      fc = one+imin

      !  Sum the terms corresponding to odd values of index i

      call grid( ni, nj, fc, gc, bfi(1:ni), bfj(1:nj), poi(1:ni), poj(1:nj), xx, yy, cdfx, iflag )
      if ( iflag /= 0 ) then
         tdnccdf = cdfx
         return
      end if
   end if

   cdfx = half*(cdfx+one-sum)
   if ( ll ) cdfx = one - cdfx
   tdnccdf = cdfx

end function tdnccdf
subroutine poisst( alamb, eps, l, nspan, v, nv, iflag )
   ! Compute the Poisson(alamb) probabilities over the range (l,k)
   ! where the total tail probability is less than eps/2, sum the
   ! probabilities and shift them to the beginning of vector v.

   implicit none

   !  Arguments
   real(kind=8) :: alamb, eps
   integer :: l, nspan, nv, iflag
   real(kind=8) :: v(1:nv)

   !  Local declarations
   real(kind=8), parameter :: zero=0.0_8, one=1.0_8, two=2.0_8, three=3.0_8
   real(kind=8) :: dal, dk, dlimit, dsum, pk, pl
   integer :: i, inc, k, nk, nl

   dlimit = one-two*eps/three
   k = int(alamb)
   l = k+1
   if ( alamb == zero ) then
      pl = one
   else
      dal = alamb
      dk = k
      pl = exp(dk*log(dal)-dal-dlgama(k+one))
   end if
   pk = alamb*pl/l
   nk = nv/2
   nl = nk+1
   dsum = zero
   do
      if ( pl < pk ) then
         nk = nk+1
         if ( nk > nv ) then
            iflag = 3
            return
         end if
         v(nk) = pk
         dsum =dsum + pk
         k = k+1
         if ( dsum >= dlimit ) exit
         pk = alamb*pk/(k+1)
      else
         nl = nl-1
         v(nl) = pl
         dsum = dsum + pl
         l = l-1
         if ( dsum >= dlimit ) exit
         pl = l*pl/alamb
      endif
   end do
   inc = nl-1
   do i = nl, nk
      v(i-inc) = v(i)
   end do
   nspan = nk-inc

end subroutine poisst

subroutine edget( nk, fc, gc, xx, yy, bfk, cdfx, poi, poj, iflag, l )
   ! Compute the Beta cdf's by a recurrence relation along the edges
   ! i=imin and j=jmin of a grid. The corresponding components of
   ! the F'' cdf are included in the summation. Terms which might
   ! cause underflow are set to zero.

   implicit none

   !  Arguments
   real(kind=8) :: fc, gc, xx, yy, cdfx
   integer :: nk, iflag, l
   real(kind=8) :: bfk(1:nk), poi(1:nk), poj(1:1)

   !  Local declarations
   real(kind=8), external :: betacdf
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: deuflo=-706.893_8
   real(kind=8) :: darg, di, dk, fd, fk
   integer :: i, j, k, kflag

   fd = fc-one
   k = max(l,min(nk,int((gc-one)*xx/yy-fd)))
   fk = fd+k
   bfk(k) = betacdf( xx,fk,gc,iflag )
   if ( iflag /= 0 ) then
      iflag = 2
      return
   end if
   if ( l == 1 ) bfk(k) = one-bfk(k)
   if ( nk /= 1 ) then
      darg = fk*log(xx)+gc*log(yy)-log(fk)+dlgama(fk+gc)-dlgama(fk)-dlgama(gc)
      if ( darg < deuflo ) then
         dk = zero
      else
         dk = exp(darg)*(-one)**l
      end if
      if ( k < nk ) then
         bfk(k+1) = bfk(k)-dk
         di = dk
         kflag = 1
         do i = k+1, nk-1
            if ( kflag == 1 ) then
               di = di*(fd+gc+(i-1))*xx/(fd+i)
               if ( dk+di == dk ) then
                  kflag = 0
                  di = zero
               end if
            end if
            bfk(i+1) = bfk(i)-di
         end do
      end if
      di = dk
      kflag = 1
      do i = k-1, l, -1
         if ( kflag == 1 ) then
            di = di*(fc+i)/((fd+gc+i)*xx)
            if ( dk+di == dk ) then
               kflag = 0
               di = zero
            end if
         end if
         bfk(i) = bfk(i+1)+di
      end do
   end if
   do i = l, nk
      cdfx = cdfx + poi(i)*poj(1)*bfk(i)
   end do

end subroutine edget

subroutine grid( ni, nj, fc, gc, bfi, bfj, poi, poj, xx, yy, cdfx, iflag )
   ! Compute double summation of components of the t'' c.d.f. over the
   ! grid i=imin to imax and j=jmin to jmax

   implicit none

   !  Arguments
   real(kind=8) :: fc, gc, xx, yy, cdfx
   integer :: ni, nj, iflag
   real(kind=8) :: bfi(1:ni), bfj(1:nj), poi(1:ni), poj(1:nj)

   !  Local declarations
   integer :: i, j

   !  Compute Beta cdf by recurrence when i=imin and j=jmin to jmax

   call edget( nj, gc, fc, yy, xx, bfj(1:nj), cdfx, poj(1:nj), poi(1:1), iflag, 1 )
   if ( ni <= 1 .or. iflag /= 0 ) return

   !  Compute Beta cdf by recurrence when j=jmin, i=imin to imax

   bfi(1) = bfj(1)
   call edget( ni, fc, gc, xx, yy, bfi(1:ni), cdfx, poi(1:ni), poj(1:1), iflag, 2 )
   if ( nj <= 1 .or. iflag /= 0 ) return

   !  Compute Beta cdf by recurrence when i>imin and j>jmin

   do i = 2, ni
      bfj(1) = bfi(i)
      do j = 2, nj
         bfj(j) = xx*bfj(j) + yy*bfj(j-1)
         cdfx = cdfx + poi(i)*poj(j)*bfj(j)
      end do
   end do

end subroutine grid
