function ksi2d( x, p, q, a2, err, maxitr, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability density at point X of a random variable
!     distributed according to the ksi-square distribution with P and Q
!     degrees of freedom and A2 eccentricity parameter.
!
!     X     - Input . Value of the variable          (X >= 0) - Real
!     P     - Input . First degrees of freedom       (P >  0) - Real
!     Q     - Input . Second   "    "     "          (Q >  0) - Real
!                     N.B. Although the function is defined
!                     for any positive Q, it is wise to
!                     take Q >= P
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     ERR   - Input . Maximum relative error required         - Real
!                     (stopping criterion)  (1e-12 < err < 1)
!     MAXITR- Input . Maximum number of iterations            - Integer
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then KSI2D = -1.0)
!                     2 = maximum number of iterations reached
!                         (then KSI2D = value at last iteration)
!
!     External functions called:
!       DLGAMA (Log(Gamma(.)) if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  LOG  SQRT  (and DLGAMA if available)
!
!     Algorithm by J-L. Guigues (1981)
!
!     J. Poitevineau (CNRS-URA1201)
!     December 03, 1986   (first version 12/14/84)
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ksi2d

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, p, q, a2, err
   integer, intent(in) :: maxitr
   integer, intent(out) :: ier

   !  local declarations
   !  ------------------

!! real(kind=8), external :: dlgama

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: two=one+one
   real(kind=8), parameter :: dlgam5=0.5723649429247000_8    ! =log(gamma(0.5))
   real(kind=8), parameter :: twol=0.6931471805599453_8      ! =log(2)
   real(kind=8), parameter :: eps=0.223e-15_8, explower=-706.893_8
   real(kind=8), parameter :: xinfinity=1.0e300_8
   real(kind=8), parameter :: relerr=1.0e-14_8 ! Relative error assumed
                                               ! in recurrence calculations

   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     explower = minimum valid argument for the exponential function
   !             (i.e. log(tinyr))
   !     xinfinity = stands for +infinity

   real(kind=8) :: axql, dj, pa, pb, phitl, pq2, p2, q2, uk, ukl, uphi, usum, xkl
   integer :: j

   !--------------------------------------------------------------------

   !  Test for valid input arguments

   if ( x < zero .or. a2 < zero .or. p <= zero .or. q <= zero  &
      .or. err >= one .or. err <= 1.0e-12_8 ) then
      ier   = 1
      ksi2d = -one
      return
   end if

   ier = 0

   !  Define constants

   p2  = p * half
   q2  = q * half
   pq2 = p2 + q2

   !  Case p = 1

   if ( abs(p-one) <= eps ) then
      if ( x > eps ) then
         ksi2d = zero
         pa = -twol - half*log(q*x) +                        &
               dlgama(half+q2) - dlgam5 - dlgama(q2) +       &
               (q2+half)*log(q/((q+a2+x)-(two*sqrt(a2*x))))
         pb = -twol - half*log(q*x) +                        &
               dlgama(half+q2) - dlgam5 - dlgama(q2) +       &
               (q2+half)*log(q/((q+a2+x)+(two*sqrt(a2*x))))
         if ( pa >= explower ) ksi2d = exp(pa)
         if ( pb >= explower ) ksi2d = ksi2d + exp(pb)
      else
         ksi2d = xinfinity
      end if
      return
   end if

   ksi2d = zero

   if ( abs(p2-one) > eps ) then

      !  Case p /= 2

      if ( x <= eps ) return
      xkl = dlgama(pq2)-dlgama(p2)-dlgama(q2)+q2*log(q)+(p2-one)*log(x)-pq2*log(a2+q+x)
      if ( xkl >= explower ) ksi2d = exp(xkl)
      !  Case a2 = 0
      if ( a2 <= eps ) return
   else

      !  Case p = 2

      xkl = dlgama(pq2) - dlgama(q2) + q2*log(q) - pq2*log(a2+q+x)
      if ( xkl >= explower ) ksi2d = exp(xkl)
      !  Case a2 = 0 or x = 0
      if ( a2 <= eps .or. x <= eps ) return
   end if

   !  General case (iterations)

   axql = log( a2*x /((a2+q+x)*(a2+q+x)) )
   ukl  = xkl
   uk   = ksi2d
   usum = zero

   !  Iteration loop

   do j = 0, maxitr
      dj    = j
      usum  = usum + dj*uk
      phitl = log((two*dj+pq2+one)*(two*dj+pq2)/((dj+one)*(dj+p2))) + axql
      ukl   = phitl + ukl
      uk    = zero
      if ( ukl >= explower ) uk = exp(ukl)
      !  Check accuracy (stopping rule)
      if ( phitl < zero ) then
         uphi = one
         if ( phitl >= explower ) uphi = uphi - exp(phitl)
         !  the end ?
         if ( (two*relerr*usum+uk/uphi)/ksi2d < err ) return
      end if
      ksi2d = ksi2d + uk
   end do

   !  Maximum number of iterations is reached

   ier = 2

end function ksi2d
