function binompmf( m, n, p, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the binomial distribution with sample size N and
!     parameter P, is equal to M (probability mass function).
!
!     M     - Input . Value of the variable (M >= 0 )         - Integer
!     N     - Input . Sample size (N > 0)                     - Integer
!     P     - Input . parameter of the binomial (0 <= P <= 1) - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!
!     External functions called:
!       (DLGAMA if not in Fortran library)
!
!     Fortran functions called:
!       DLGAMA
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: binompmf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: p
   integer, intent(in) :: m, n
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8), parameter :: explower=-706.893_8
   !     explower = minimum valid argument for the exponential function

   real(kind=8) :: t

   !--------------------------------------------------------------------

   binompmf = zero
   ier = 0

   !  Test for valid input arguments

   if ( n <= 0 .or. m < 0 .or. m > n .or. p < zero .or. p > one ) then
      ier = 1
      return
   end if

   !  Special cases

   if ( p == zero ) then
      if ( m == 0 ) binompmf = one
      return
   else if ( p == one ) then
      if ( m == n ) binompmf = one
      return
   end if

   !  General case

   if ( m == 0 ) then
      t = n*log(one-p)
   else if ( m == n ) then
      t = n*log(p)
   else
      t = dlgama(n+one) - dlgama(m+one) - dlgama(n-m+one) + m*log(p) + (n-m)*log(one-p)
   end if
   if ( t > explower ) binompmf = exp(t)

end function binompmf
