function nbinompmf( k, r, p, ier )

!-----------------------------------------------------------------------
!
!     Computes the probability that a random variable distributed
!     according to the negative binomial distribution with R successes
!     and probability of success P, is equal to K
!
!     K     - Input . Value of the variable: number of        - Integer
!                     failures (K >= 0)
!     R     - Input . Number of successes (R > 0)             - Integer
!     P     - Input . Probability of success (0<= P <=1)      - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!
!     External functions called:
!       (DLGAMA if not in Fortran library)
!
!     Fortran functions called:
!       DLGAMA
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: nbinompmf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: p
   integer, intent(in) :: k, r
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8), parameter :: explower=-706.893_8
   !     explower = minimum valid argument for the exponential function

   real(kind=8) :: t, xr

   !--------------------------------------------------------------------

   nbinompmf = zero
   ier = 0

   !  Test for valid input arguments

   if ( r <= 0 .or. k < 0 .or. p < zero .or. p > one ) then
      ier = 1
      nbinompmf = -one
      return
   end if

   !  Special cases

   if ( p == zero ) then
      return
   else if ( p == one ) then
      if ( k == 0 ) nbinompmf = one
      return
   end if

   !  General case

   xr = r
   if ( k == 0 ) then
      t = xr*log(p)
   else
      t = dlgama(xr+k) - dlgama(k+one) - dlgama(xr) + xr*log(p) + k*log(one-p)
   end if
   if ( t > explower ) nbinompmf = exp(t)

end function nbinompmf
