function ibinomc( pbin, n, p, ier )

!-----------------------------------------------------------------------
!
!     Inverse of the continued binomial cdf.
!     Given that a random variable is distributed according to the
!     continued binomial distribution with sample size N and probability
!     parameter P, it returns the value such that the probability
!     that the random variable is less than or equal to that value is
!     at least PBIN.
!
!     PBIN  - Input . Input probability (cdf) (0<= PBIN <=1)  - Real
!     N     - Input . Sample size (N > 0)                     - Integer
!     P     - Input . Probability of success (0<= P <=1)      - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     2 = maximum number of iterations reached
!                     3 = result out of limits
!                         (ibinomc < -1 or ibinomc > n)
!
!     External functions called:
!       BINOMCDF  BINOMPMF  IBINOM
!
!     Fortran functions called:
!       NONE
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ibinomc

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: pbin, p
   integer, intent(in) :: n
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: binomcdf, binompmf
   integer :: ibinom

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8) :: t
   integer :: i

   !--------------------------------------------------------------------

   ibinomc = -one
   ier = 1

   !  Test for valid input arguments

   if ( n <= 0 .or. pbin < zero .or. pbin > one    &
               .or. p < zero .or. p > one ) return

   if ( pbin == one ) then
      ibinomc = n
      return
   end if
   i = ibinom( pbin, n, p, ier )
   if ( ier /= 0 ) return
   t = binompmf( i, n, p, ier )
   if ( ier /= 0 ) return
   if ( t > zero ) then
      ibinomc = i - (binomcdf( i, n, p, ier ) - pbin)/t
   else
      ibinomc = i
   end if

end function ibinomc
