function ibinomcf( f, a, b, u, ier )

!-----------------------------------------------------------------------
!
!     Inverse of the fiducial continued binomial cdf.
!     Given that a random variable is distributed according to the
!     fiducial continued binomial distribution with parameters A, B, U,
!     it returns the value such that the probability that the random
!     variable is less than or equal to that value is F, up to a
!     precision parameter.
!
!     F     - Input . Input probability (cdf) (0<= F <=1)     - Real
!     A     - Input . parameter: number of "positive" events  - Real
!                     (A > 0)
!     B     - Input . parameter: number of "negative" events  - Real
!                     (B > 0)
!     U     - Input . parameter                               - Real
!                     (0 <= U <= 1)
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     2 = maximum number of iterations reached
!                     3 = result out of limits
!                         (ibinomcf < -1 or ibinomcf > n)
!
!     External functions called:
!       BINOMCFCDF
!
!     Fortran functions called:
!       ABS
!
!-----------------------------------------------------------------------
!
!     Algorithm:
!     Initial solution: 1/2.
!     Then, an empirical upper bound is found by incrementing the mean
!     by the standard deviation as long as needed.
!     Then the bounds are iteratively modified by a 2-step process
!     (let Li and Ui the lower and upper bounds at iteration no. i):
!        step1: solution Xi = linear interpolation on interval (Li,Ui)
!        step2: solution Yi = middle of (Xi,Ui) (or (Li,Xi) according
!                             to whether BINOMCFCDF(Xi) < F or > F),
!               Li+1 = Li (or Yi),  Ui+1 = Yi (or Ui)
!
!     Of course at each stage, whenever BINOMCFCDF is called, the result
!     is compared to F and the process is stopped if the solution fits.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ibinomcf

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: f, a, b, u
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: binomcfcdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: delta=1.0e-12_8
   real(kind=8), parameter :: xsup=one
   real(kind=8), parameter :: maxit=2000

   real(kind=8) :: xa, xm, xz, y, ya, yz
   integer :: iok, it

   !--------------------------------------------------------------------

   ibinomcf = zero
   ier = 0

   !  Test for valid input arguments

   if ( f < zero .or. f > one .or. a < zero .or. b < zero .or. u < zero ) then
      ier    = 1
      ibinomcf = -xsup
      return
   end if

   !  f near 0 or 1

   if ( f <= delta ) return
   if ( f >= one-delta ) then
      ibinomcf = xsup
      return
   end if

   !  Initial solution xm

   xm = half
   y = binomcfcdf( xm, a, b, u, iok )
   if ( iok /= 0 ) goto 30
   if ( abs(f-y) <= delta ) then
      ibinomcf = xm
      return
   end if

   !  Get lower (xa) and upper (xz) bounds for x

   if ( f < y ) then
      !  Get an upper bound
      xz = xm
      yz = y
      xa = zero
      ya = zero
   else
      ! Get a lower bound
      xa = xm
      ya = y
      xz = xsup
      yz = one
   end if

   !  Iteration loop

   do it = 1, maxit

      !  1st step: solution xm = linear interpolation on (xa, xz)

      !  Note that there's no protection for xz=xa, f=ya or yz=ya
      !  in the following statement because it should not occur...

      xm = xa + (xz-xa)*(f-ya)/(yz-ya)

      y  = binomcfcdf( xm, a, b, u, iok )
      if ( iok /= 0 ) goto 30
      if ( abs(f-y) <= delta ) then
         ibinomcf = xm
         return
      end if
      if ( f < y ) then
         xz = xm
         yz = y
      else
         xa = xm
         ya = y
      end if

      !  2nd step: solution xm = middle of updated (xa, xz)

      xm = (xa+xz) * half
      y  = binomcfcdf( xm, a, b, u, iok )
      if ( iok /= 0 ) goto 30
      if ( abs(f-y) <= delta ) then
         ibinomcf = xm
         return
      end if
      if ( f < y ) then
         xz = xm
         yz = y
      else
         xa = xm
         ya = y
      end if

   end do

   !  Maximum number of iterations is reached

   ier = 2
   ibinomcf = xm
   return

   30 continue
   !  Error in  binomcfcdf function

   ier = -iok
   ibinomcf = -xsup

end function ibinomcf
