function bessel_i( x, no, alpha, ier )

!-----------------------------------------------------------------------
!
!     Computes the modified Bessel function of the first kind,
!     Ino+alpha (x), for non-negative argument x, and non-negative
!     order no+alpha
!
!     X     - Input . Argument (0 <= X < exparg see below)    - Real
!     NO    - Input . Integer part of order (NO >= 0)         - Integer
!     ALPHA - Input . Fractional part of order                - Real
!                     (0 <= ALPHA < 1)
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     <0= Requested function value could not
!                         be calculated accurately
!
!     Fortran functions called:
!       EXP  DGAMMA  INT  MAX  MIN  REAL  SQRT
!
!     From Netlib library subroutine
!
!-----------------------------------------------------------------------
!
!  This routine calculates Bessel functions I SUB(N+ALPHA) (X)
!  for non-negative argument X, and non-negative order N+ALPHA,
!  with or without exponential scaling.
!
!
! Explanation of variables in the calling sequence
!
! X     - Working precision non-negative real argument for which
!         I's or exponentially scaled I's (I*EXP(-X))
!         are to be calculated.  If I's are to be calculated,
!         X must be less than EXPARG (see below).
! ALPHA - Working precision fractional part of order for which
!         I's or exponentially scaled I's (I*EXP(-X)) are
!         to be calculated.  0 .LE. ALPHA .LT. 1.0.
! NB    - Integer number of functions to be calculated, NB .GT. 0.
!         The first function calculated is of order ALPHA, and the
!         last is of order (NB - 1 + ALPHA).
! IZE   - Integer type.  IZE = 1 if unscaled I's are to calculated,
!         and 2 if exponentially scaled I's are to be calculated.
! B     - Working precision output vector of length NB.  If the routine
!         terminates normally (NCALC=NB), the vector B contains the
!         functions I(ALPHA,X) through I(NB-1+ALPHA,X), or the
!         corresponding exponentially scaled functions.
! NCALC - Integer output variable indicating possible errors.
!         Before using the vector B, the user should check that
!         NCALC=NB, i.e., all orders have been calculated to
!         the desired accuracy.  See error returns below.
!
!*******************************************************************
!
! Explanation of machine-dependent constants
!
!   beta   = Radix for the floating-point system
!   minexp = Smallest representable power of beta
!   maxexp = Smallest power of beta that overflows
!   it     = Number of bits in the mantissa of a working precision
!            variable
!   NSIG   = Decimal significance desired.  Should be set to
!            INT(LOG10(2)*it+1).  Setting NSIG lower will result
!            in decreased accuracy while setting NSIG higher will
!            increase CPU time without increasing accuracy.  The
!            truncation error is limited to a relative error of
!            T=.5*10**(-NSIG).
!   ENTEN  = 10.0 ** K, where K is the largest integer such that
!            ENTEN is machine-representable in working precision
!   ENSIG  = 10.0 ** NSIG
!   RTNSIG = 10.0 ** (-K) for the smallest integer K such that
!            K .GE. NSIG/4
!   ENMTEN = Smallest ABS(X) such that X/4 does not underflow
!   XLARGE = Upper limit on the magnitude of X when IZE=2.  Bear
!            in mind that if ABS(X)=N, then at least N iterations
!            of the backward recursion will be executed.  The value
!            of 10.0 ** 4 is used on every machine.
!   EXPARG = Largest working precision argument that the library
!            EXP routine can handle and upper limit on the
!            magnitude of X when IZE=1; approximately
!            LOG(beta**maxexp)
!
!     Approximate values for some important machines are:
!
!                        beta       minexp      maxexp       it
!
!  CRAY-1        (S.P.)    2        -8193        8191        48
!  Cyber 180/855
!    under NOS   (S.P.)    2         -975        1070        48
!  IEEE (IBM/XT,
!    SUN, etc.)  (S.P.)    2         -126         128        24
!  IEEE (IBM/XT,
!    SUN, etc.)  (D.P.)    2        -1022        1024        53
!  IBM 3033      (D.P.)   16          -65          63        14
!  VAX           (S.P.)    2         -128         127        24
!  VAX D-Format  (D.P.)    2         -128         127        56
!  VAX G-Format  (D.P.)    2        -1024        1023        53
!
!                        NSIG       ENTEN       ENSIG      RTNSIG
!
! CRAY-1        (S.P.)    15       1.0E+2465   1.0E+15     1.0E-4
! Cyber 180/855
!   under NOS   (S.P.)    15       1.0E+322    1.0E+15     1.0E-4
! IEEE (IBM/XT,
!   SUN, etc.)  (S.P.)     8       1.0E+38     1.0E+8      1.0E-2
! IEEE (IBM/XT,
!   SUN, etc.)  (D.P.)    16       1.0D+308    1.0D+16     1.0D-4
! IBM 3033      (D.P.)     5       1.0D+75     1.0D+5      1.0D-2
! VAX           (S.P.)     8       1.0E+38     1.0E+8      1.0E-2
! VAX D-Format  (D.P.)    17       1.0D+38     1.0D+17     1.0D-5
! VAX G-Format  (D.P.)    16       1.0D+307    1.0D+16     1.0D-4
!
!                         ENMTEN      XLARGE   EXPARG
!
! CRAY-1        (S.P.)   1.84E-2466   1.0E+4    5677
! Cyber 180/855
!   under NOS   (S.P.)   1.25E-293    1.0E+4     741
! IEEE (IBM/XT,
!   SUN, etc.)  (S.P.)   4.70E-38     1.0E+4      88
! IEEE (IBM/XT,
!   SUN, etc.)  (D.P.)   8.90D-308    1.0D+4     709
! IBM 3033      (D.P.)   2.16D-78     1.0D+4     174
! VAX           (S.P.)   1.17E-38     1.0E+4      88
! VAX D-Format  (D.P.)   1.17D-38     1.0D+4      88
! VAX G-Format  (D.P.)   2.22D-308    1.0D+4     709
!
!*******************************************************************
!
! Error returns
!
!  In case of an error,  NCALC .NE. NB, and not all I's are
!  calculated to the desired accuracy.
!
!  NCALC .LT. 0:  An argument is out of range. For example,
!     NB .LE. 0, IZE is not 1 or 2, or IZE=1 and ABS(X) .GE. EXPARG.
!     In this case, the B-vector is not calculated, and NCALC is
!     set to MIN0(NB,0)-1 so that NCALC .NE. NB.
!
!  NB .GT. NCALC .GT. 0: Not all requested function values could
!     be calculated accurately.  This usually occurs because NB is
!     much larger than ABS(X).  In this case, B(N) is calculated
!     to the desired accuracy for N .LE. NCALC, but precision
!     is lost for NCALC .LT. N .LE. NB.  If B(N) does not vanish
!     for N .GT. NCALC (because it is too small to be represented),
!     and B(N)/B(NCALC) = 10**(-K), then only the first NSIG-K
!     significant figures of B(N) can be trusted.
!
! Intrinsic functions required are:
!
!     DBLE, EXP, DGAMMA, GAMMA, INT, MAX, MIN, REAL, SQRT
!
! Acknowledgement
!
!  This program is based on a program written by David J.
!  Sookne (2) that computes values of the Bessel functions J or
!  I of real argument and integer order.  Modifications include
!  the restriction of the computation to the I Bessel function
!  of non-negative real argument, the extension of the computation
!  to arbitrary positive order, the inclusion of optional
!  exponential scaling, and the elimination of most underflow.
!  An earlier version was published in (3).
!
! References: "A Note on Backward Recurrence Algorithms," Olver,
!              F. W. J., and Sookne, D. J., Math. Comp. 26, 1972,
!              pp 941-947.
!
!             "Bessel Functions of Real Argument and Integer Order,"
!              Sookne, D. J., NBS Jour. of Res. B. 77B, 1973, pp
!              125-132.
!
!             "ALGORITHM 597, Sequence of Modified Bessel Functions
!              of the First Kind," Cody, W. J., Trans. Math. Soft.,
!              1983, pp. 242-245.
!
!  Latest modification: May 30, 1989
!
!  Modified by: W. J. Cody and L. Stoltz
!               Applied Mathematics Division
!               Argonne National Laboratory
!               Argonne, IL  60439
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: bessel_i

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, alpha
   integer, intent(in) :: no
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

      !  Mathematical constants
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8, const=1.585_8
      !  Machine dependent parameters
   real(kind=8), parameter :: exparg=709.0_8, xlarge=1.0e4_8
   real(kind=8), parameter :: enmten=8.9e-308_8, enten=1.0e308_8, ensig=1.0e16_8, rtnsig=1.0e-4_8
   integer, parameter :: nsig=16

   real(kind=8) :: em, empal, emp2al, en, halfx, p, plast, pold, psave, psavel
   real(kind=8) :: sum, tempa, tempb, tempc, test, tover
   integer :: k, l, magx, n, nb, nbmx, ncalc, nend, nstart
   real(kind=8) :: b(no+1)

   !--------------------------------------------------------------------

   bessel_i = zero
   nb = no + 1
   if ( nb <= 0 .or. alpha < zero .or. alpha >= one .or. x > exparg .or. x < zero ) then
      ier = 1
      return
   end if
      !------------------------------------------
      ! Use 2-term ascending series for small x
      !------------------------------------------
   ncalc = nb
   magx = int(x)
   if ( x >= rtnsig ) then
         !--------------------------------------------------------
         ! Initialize the forward sweep, the p-sequence of Olver
         !--------------------------------------------------------
      nbmx = nb-magx
      n = magx+1
      en = n+n + (alpha+alpha)
      plast = one
      p = en / x
         !--------------------------------------
         ! Calculate general significance test
         !--------------------------------------
      test = ensig + ensig
      if ( 2*magx > 5*nsig ) then
            test = sqrt(test*p)
         else
            test = test / const**magx
      end if
      if ( nbmx >= 3 ) then
            !--------------------------------------------------------------------
            ! Calculate p-sequence until n = nb-1.  Check for possible overflow
            !--------------------------------------------------------------------
         tover = enten / ensig
         nstart = magx+2
         nend = nb - 1
         do k = nstart, nend
            n = k
            en = en + two
            pold = plast
            plast = p
            p = en * plast/x + pold
            if ( p > tover ) then
                  !------------------------------------------------------------
                  ! To avoid overflow, divide p-sequence by tover.  Calculate
                  ! p-sequence until abs(p) > 1
                  !------------------------------------------------------------
               tover = enten
               p = p / tover
               plast = plast / tover
               psave = p
               psavel = plast
               nstart = n + 1
               do
                  n = n + 1
                  en = en + two
                  pold = plast
                  plast = p
                  p = en * plast/x + pold
                  if ( p > one ) exit
               end do
               tempb = en / x
                  !---------------------------------------------------------
                  ! Calculate backward test, and find ncalc, the highest n
                  ! such that the test is passed.
                  !---------------------------------------------------------
               test = pold*plast / ensig
               test = test*(half-half/(tempb*tempb))
               p = plast * tover
               n = n - 1
               en = en - two
               nend = min0(nb,n)
               do l = nstart, nend
                  ncalc = l
                  pold = psavel
                  psavel = psave
                  psave = en * psavel/x + pold
                  if ( psave*psavel > test ) then
                     ncalc = ncalc - 1
                     goto 10
                  end if
               end do
               ncalc = nend
               goto 10
            end if
         end do
         n = nend
         en = n+n + (alpha+alpha)
            !---------------------------------------------------
            ! Calculate special significance test for nbmx > 2
            !---------------------------------------------------
         test = max(test,sqrt(plast*ensig)*sqrt(p+p))
      end if
         !------------------------------------------------------
         ! Calculate p-sequence until significance test passed
         !------------------------------------------------------
      do
         n = n + 1
         en = en + two
         pold = plast
         plast = p
         p = en * plast/x + pold
         if ( p >= test ) exit
      end do
   10 continue
         !--------------------------------------------------------------
         ! Initialize the backward recursion and the normalization sum
         !--------------------------------------------------------------
      n = n + 1
      en = en + two
      tempb = zero
      tempa = one / p
      em = n - one
      empal = em + alpha
      emp2al = (em - one) + (alpha + alpha)
      sum = tempa * empal * emp2al / em
      nend = n - nb
      if ( nend < 0 ) then
            !------------------------------------------------------
            ! n < nb, so store b(n) and set higher orders to zero
            !------------------------------------------------------
         b(n) = tempa
         nend = -nend
         do l = 1, nend
            b(n+l) = zero
         end do
      else
         if ( nend > 0 ) then
               !-----------------------------------------------------------
               ! Recur backward via difference equation, calculating (but
               ! not storing) b(n), until n = nb.
               !-----------------------------------------------------------
            do l = 1, nend
               n = n - 1
               en = en - two
               tempc = tempb
               tempb = tempa
               tempa = (en*tempb) / x + tempc
               em = em - one
               emp2al = emp2al - one
               if ( n == 1 ) goto 20
               if ( n == 2 ) emp2al = one
               empal = empal - one
               sum = (sum + tempa*empal) * emp2al / em
            end do
         end if
      20 continue
            !--------------
            ! Store b(nb)
            !--------------
         b(n) = tempa
         if ( nb <= 1 ) then
            sum = (sum + sum) + tempa
            goto 40
         end if
            !------------------------------
            ! Calculate and store b(nb-1)
            !------------------------------
         n = n - 1
         en = en - two
         b(n)  = (en*tempa) / x + tempb
         if ( n == 1 ) goto 30
         em = em - one
         emp2al = emp2al - one
         if ( n == 2 ) emp2al = one
         empal = empal - one
         sum = (sum + b(n)*empal) * emp2al / em
      end if
      nend = n - 2
      if ( nend > 0 ) then
            !----------------------------------------------------------------
            ! Calculate via difference equation and store b(n), until n = 2
            !----------------------------------------------------------------
         do l = 1, nend
            n = n - 1
            en = en - two
            b(n) = (en*b(n+1)) / x +b(n+2)
            em = em - one
            emp2al = emp2al - one
            if ( n == 2 ) emp2al = one
            empal = empal - one
            sum = (sum + b(n)*empal) * emp2al / em
         end do
      end if
         !-----------------
         ! Calculate b(1)
         !-----------------
      b(1) = two*empal*b(2) / x + b(3)
   30 continue
      sum = (sum + sum) + b(1)
   40 continue
         !-------------------------------------
         ! Normalize.  Divide all b(n) by sum
         !-------------------------------------
      if ( alpha /= zero ) sum = sum * dgamma(one+alpha) * (x*half)**(-alpha)
      sum = sum * exp(-x)
      tempa = enmten
      if ( sum > one ) tempa = tempa * sum
      do n = 1, nb
         if ( b(n) < tempa ) b(n) = zero
         b(n) = b(n) / sum
      end do
      ier = ncalc - nb
      if ( ier == 0 ) bessel_i = b(nb)
      return
   else
         !----------------------------------------
         ! Two-term ascending series for small x
         !----------------------------------------
      tempa = one
      empal = one + alpha
      halfx = zero
      if ( x > enmten ) halfx = half * x
      if ( alpha /= zero ) tempa = halfx**alpha /dgamma(empal)
      tempb = zero
      if ( (x+one) > one ) tempb = halfx * halfx
      b(1) = tempa + tempa*tempb / empal
      if ( (x /= zero) .and. (b(1) == zero) ) ncalc = 0
      if ( nb > 1 ) then
         if ( x == zero ) then
            do n = 2, nb
               b(n) = zero
            end do
         else
               !-----------------------------------
               ! Calculate higher-order functions
               !-----------------------------------
            tempc = halfx
            tover = (enmten + enmten) / x
            if ( tempb /= zero ) tover = enmten / tempb
            do n = 2, nb
               tempa = tempa / empal
               empal = empal + one
               tempa = tempa * tempc
               if ( tempa <= tover*empal ) tempa = zero
               b(n) = tempa + tempa*tempb / empal
               if ( (b(n) == zero) .and. (ncalc > n) ) ncalc = n-1
            end do
         end if
      end if
   end if
   ier = ncalc - nb
   if ( ier == 0 ) bessel_i = b(nb)

end function bessel_i
