function bessel_k( x, no, alpha, ier )

!-----------------------------------------------------------------------
!
!     Computes the modified Bessel function of the second kind,
!     Kno+alpha (x), for positive argument x, and non-negative
!     order no+alpha
!
!     X     - Input . Argument (0 < X < xmax see below)       - Real
!     NO    - Input . Integer part of order (NO >= 0)         - Integer
!     ALPHA - Input . Fractional part of order                - Real
!                     (0 <= ALPHA < 1)
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     <0= Requested function value could not
!                         be calculated accurately
!
!     Fortran functions called:
!       ABS  AINT  EXP  INT  LOG  MAX  MIN  SINH  SQRT
!
!     From Netlib library subroutine
!
!-----------------------------------------------------------------------
!
!  This FORTRAN 77 routine calculates modified Bessel functions
!  of the second kind, K SUB(N+ALPHA) (X), for non-negative
!  argument X, and non-negative order N+ALPHA, with or without
!  exponential scaling.
!
!  Explanation of variables in the calling sequence
!
!  Description of output values ..
!
! X     - Working precision non-negative real argument for which
!         K's or exponentially scaled K's (K*EXP(X))
!         are to be calculated.  If K's are to be calculated,
!         X must not be greater than XMAX (see below).
! ALPHA - Working precision fractional part of order for which
!         K's or exponentially scaled K's (K*EXP(X)) are
!         to be calculated.  0 .LE. ALPHA .LT. 1.0.
! NB    - Integer number of functions to be calculated, NB .GT. 0.
!         The first function calculated is of order ALPHA, and the
!         last is of order (NB - 1 + ALPHA).
! IZE   - Integer type.  IZE = 1 if unscaled K's are to be calculated,
!         and 2 if exponentially scaled K's are to be calculated.
! BK    - Working precision output vector of length NB.  If the
!         routine terminates normally (NCALC=NB), the vector BK
!         contains the functions K(ALPHA,X), ... , K(NB-1+ALPHA,X),
!         or the corresponding exponentially scaled functions.
!         If (0 .LT. NCALC .LT. NB), BK(I) contains correct function
!         values for I .LE. NCALC, and contains the ratios
!         K(ALPHA+I-1,X)/K(ALPHA+I-2,X) for the rest of the array.
! NCALC - Integer output variable indicating possible errors.
!         Before using the vector BK, the user should check that
!         NCALC=NB, i.e., all orders have been calculated to
!         the desired accuracy.  See error returns below.
!
!*******************************************************************
!
! Explanation of machine-dependent constants
!
!   beta   = Radix for the floating-point system
!   minexp = Smallest representable power of beta
!   maxexp = Smallest power of beta that overflows
!   EPS    = The smallest positive floating-point number such that
!            1.0+EPS .GT. 1.0
!   XMAX   = Upper limit on the magnitude of X when IZE=1;  Solution
!            to equation:
!               W(X) * (1-1/8X+9/128X**2) = beta**minexp
!            where  W(X) = EXP(-X)*SQRT(PI/2X)
!   SQXMIN = Square root of beta**minexp
!   XINF   = Largest positive machine number; approximately
!            beta**maxexp
!   XMIN   = Smallest positive machine number; approximately
!            beta**minexp
!
!     Approximate values for some important machines are:
!
!                          beta       minexp      maxexp      EPS
!
!  CRAY-1        (S.P.)      2        -8193        8191    7.11E-15
!  Cyber 180/185
!    under NOS   (S.P.)      2         -975        1070    3.55E-15
!  IEEE (IBM/XT,
!    SUN, etc.)  (S.P.)      2         -126         128    1.19E-7
!  IEEE (IBM/XT,
!    SUN, etc.)  (D.P.)      2        -1022        1024    2.22D-16
!  IBM 3033      (D.P.)     16          -65          63    2.22D-16
!  VAX           (S.P.)      2         -128         127    5.96E-8
!  VAX D-Format  (D.P.)      2         -128         127    1.39D-17
!  VAX G-Format  (D.P.)      2        -1024        1023    1.11D-16
!
!                         SQXMIN       XINF        XMIN      XMAX
!
! CRAY-1        (S.P.)  6.77E-1234  5.45E+2465  4.59E-2467 5674.858
! Cyber 180/855
!   under NOS   (S.P.)  1.77E-147   1.26E+322   3.14E-294   672.788
! IEEE (IBM/XT,
!   SUN, etc.)  (S.P.)  1.08E-19    3.40E+38    1.18E-38     85.337
! IEEE (IBM/XT,
!   SUN, etc.)  (D.P.)  1.49D-154   1.79D+308   2.23D-308   705.342
! IBM 3033      (D.P.)  7.35D-40    7.23D+75    5.40D-79    177.852
! VAX           (S.P.)  5.42E-20    1.70E+38    2.94E-39     86.715
! VAX D-Format  (D.P.)  5.42D-20    1.70D+38    2.94D-39     86.715
! VAX G-Format  (D.P.)  7.46D-155   8.98D+307   5.57D-309   706.728
!
!*******************************************************************
!
! Error returns
!
!  In case of an error, NCALC .NE. NB, and not all K's are
!  calculated to the desired accuracy.
!
!  NCALC .LT. -1:  An argument is out of range. For example,
!       NB .LE. 0, IZE is not 1 or 2, or IZE=1 and ABS(X) .GE.
!       XMAX.  In this case, the B-vector is not calculated,
!       and NCALC is set to MIN0(NB,0)-2  so that NCALC .NE. NB.
!  NCALC = -1:  Either  K(ALPHA,X) .GE. XINF  or
!       K(ALPHA+NB-1,X)/K(ALPHA+NB-2,X) .GE. XINF.  In this case,
!       the B-vector is not calculated.  Note that again
!       NCALC .NE. NB.
!
!  0 .LT. NCALC .LT. NB: Not all requested function values could
!       be calculated accurately.  BK(I) contains correct function
!       values for I .LE. NCALC, and contains the ratios
!       K(ALPHA+I-1,X)/K(ALPHA+I-2,X) for the rest of the array.
!
! Intrinsic functions required are:
!
!     ABS, AINT, EXP, INT, LOG, MAX, MIN, SINH, SQRT
!
! Acknowledgement
!
!  This program is based on a program written by J. B. Campbell
!  (2) that computes values of the Bessel functions K of real
!  argument and real order.  Modifications include the addition
!  of non-scaled functions, parameterization of machine
!  dependencies, and the use of more accurate approximations
!  for SINH and SIN.
!
! References: "On Temme's Algorithm for the Modified Bessel
!              Functions of the Third Kind," Campbell, J. B.,
!              TOMS 6(4), Dec. 1980, pp. 581-586.
!
!             "A FORTRAN IV Subroutine for the Modified Bessel
!              Functions of the Third Kind of Real Order and Real
!              Argument," Campbell, J. B., Report NRC/ERB-925,
!              National Research Council, Canada.
!
!  Latest modification: May 30, 1989
!
!  Modified by: W. J. Cody and L. Stoltz
!               Applied Mathematics Division
!               Argonne National Laboratory
!               Argonne, IL  60439
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: bessel_k

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, alpha
   integer, intent(in) :: no
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

      !  Mathematical constants
      !    a = log(2.d0) - Euler's constant
      !    d = sqrt(2.d0/pi)
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8, four=4.0_8
   real(kind=8), parameter :: a=0.11593151565841244881_8, d=0.797884560802865364_8
   real(kind=8), parameter :: tinyx=1.0e-10_8
      !  Machine dependent parameters
   real(kind=8), parameter :: eps=2.22e-16_8, sqxmin=1.49e-154_8, xinf=1.79e+308_8
   real(kind=8), parameter :: xmin=2.23e-308_8, xmax=705.342_8

   real(kind=8) :: blpha, bk1, bk2, c, dm, d1, d2, d3, enu, ex
   real(kind=8) :: f0, f1, f2, p0, q0, ratio, twonu, twox, t1, t2, wminf, x2by4
   integer :: i, iend, itemp, j, k, m, mplus1, nb, ncalc
   real(kind=8) :: bk(no+1)

      !  p, q - approximation for log(gamma(1+alpha))/alpha
      !                                         + Euler's constant
      !         Coefficients converted from hex to decimal and modified
      !         by W. J. Cody, 2/26/82
      !  r, s - approximation for (1-alpha*pi/sin(alpha*pi))/(2*alpha)
      !  t    - approximation for sinh(y)/y
   real(kind=8) :: p(8)=(/  &
              0.805629875690432845_8   ,  0.204045500205365151e02_8, &
              0.157705605106676174e03_8,  0.536671116469207504e03_8, &
              0.900382759291288778e03_8,  0.730923886650660393e03_8, &
              0.229299301509425145e03_8,  0.822467033424113231_8/)
   real(kind=8) :: q(7)=(/  &
              0.294601986247850434e02_8,  0.277577868510221208e03_8, &
              0.120670325591027438e04_8,  0.276291444159791519e04_8, &
              0.344374050506564618e04_8,  0.221063190113378647e04_8, &
              0.572267338359892221e03_8/)
   real(kind=8) :: r(5)=(/  &
             -0.48672575865218401848_8   ,0.13079485869097804016e+2_8, &
             -0.10196490580880537526e+3_8,0.34765409106507813131e+3_8, &
              0.34958981245219347820e-3_8/)
   real(kind=8) :: s(4)=(/  &
             -0.25579105509976461286e+2_8,0.21257260432226544008e+3_8, &
             -0.61069018684944109624e+3_8,0.42269668805777760407e+3_8/)
   real(kind=8) :: t(6)=(/  &
              0.16125990452916363814e-9_8,0.25051878502858255354e-7_8, &
              0.27557319615147964774e-5_8,0.19841269840928373686e-3_8, &
              0.83333333333334751799e-2_8,0.16666666666666666446_8/)
   real(kind=8) :: estf(7)=(/  &
              4.18341e1_8, 7.1075_8, 6.4306_8, 4.25110e1_8, 1.35633_8, 8.45096e1_8, 2.0e1_8/)
   real(kind=8) :: estm(6)=(/  &
              5.20583e1_8, 5.7607_8, 2.7782_8, 1.44303e1_8, 1.853004e2_8, 9.3715_8/)

   !--------------------------------------------------------------------

   bessel_k = zero
   nb = no + 1
   ncalc = min(nb,0)-2
   if ( nb <= 0 .or. alpha < zero .or. alpha >= one .or. x > xmax .or. x <= zero ) then
      ier = 1
      return
   end if
   ier = 0
   ex = x
   enu = alpha
   k = 0
   if (enu < sqxmin) enu = zero
   if ( enu > half ) then
      k = 1
      enu = enu - one
   end if
   twonu = enu+enu
   iend = nb+k-1
   c = enu*enu
   d3 = -c
   if ( ex <= one ) then
         !-----------------------------------------------------
         !  Calculation of p0 = gamma(1+alpha) * (2/x)**alpha
         !                 q0 = gamma(1-alpha) * (x/2)**alpha
         !-----------------------------------------------------
      d1 = zero
      d2 = p(1)
      t1 = one
      t2 = q(1)
      do i = 2, 7, 2
         d1 = c*d1+p(i)
         d2 = c*d2+p(i+1)
         t1 = c*t1+q(i)
         t2 = c*t2+q(i+1)
      end do
      d1 = enu*d1
      t1 = enu*t1
      f1 = log(ex)
      f0 = a+enu*(p(8)-enu*(d1+d2)/(t1+t2))-f1
      q0 = exp(-enu*(a-enu*(p(8)+enu*(d1-d2)/(t1-t2))-f1))
      f1 = enu*f0
      p0 = exp(f1)
         !-----------------------
         !  Calculation of f0 =
         !-----------------------
      d1 = r(5)
      t1 = one
      do i = 1, 4
         d1 = c*d1+r(i)
         t1 = c*t1+s(i)
      end do
      if ( abs(f1) <= half ) then
         f1 = f1*f1
         d2 = zero
         do i = 1, 6
            d2 = f1*d2+t(i)
         end do
         d2 = f0+f0*f1*d2
      else
         d2 = sinh(f1)/enu
      end if
      f0 = d2-enu*d1/(t1*p0)
      if ( ex <= tinyx ) then
            !-----------------------------------------------------------
            !  x <= 1.0e-10
            !  Calculation of K(alpha,x) and x*K(alpha+1,x)/K(alpha,x)
            !-----------------------------------------------------------
         bk(1) = f0+ex*f0
         bk(1) = bk(1)-ex*bk(1)
         ratio = p0/f0
         c = ex*xinf
         if ( k /= 0 ) then
               !------------------------------------------------------------
               !  Calculation of K(alpha,x) and x*K(alpha+1,x)/K(alpha,x),
               !  alpha >= 1/2
               !------------------------------------------------------------
            ncalc = -1
            if ( bk(1) >= c/ratio ) goto 50
            bk(1) = ratio*bk(1)/ex
            twonu = twonu+two
            ratio = twonu
         end if
         ncalc = 1
         if ( nb == 1) goto 50
            !-------------------------------------------------------------------
            !  Calculate  K(alpha+l,x)/K(alpha+l-1,x),  l  =  1, 2, ... , nb-1
            !-------------------------------------------------------------------
         ncalc = -1
         do i = 2, nb
            if (ratio >= c ) goto 50
            bk(i) = ratio/ex
            twonu = twonu+two
            ratio = twonu
         end do
         ncalc = 1
         goto 20
      else
            !----------------------
            !  1.0e-10 < x <= 1.0
            !----------------------
         c = one
         x2by4 = ex*ex/four
         p0 = half*p0
         q0 = half*q0
         d1 = -one
         d2 = zero
         bk1 = zero
         bk2 = zero
         f1 = f0
         f2 = p0
         do
            d1 = d1+two
            d2 = d2+one
            d3 = d1+d3
            c = x2by4*c/d2
            f0 = (d2*f0+p0+q0)/d3
            p0 = p0/(d2-enu)
            q0 = q0/(d2+enu)
            t1 = c*f0
            t2 = c*(p0-d2*f0)
            bk1 = bk1+t1
            bk2 = bk2+t2
            if ( (abs(t1/(f1+bk1)) <= eps) .and. (abs(t2/(f2+bk2)) <= eps) ) exit
         end do
         bk1 = f1+bk1
         bk2 = two*(f2+bk2)/ex
         wminf = estf(1)*ex+estf(2)
      end if
   else if ( eps*ex > one ) then
         !---------------
         !  x > one/eps
         !---------------
      ncalc = nb
      bk1 = one / (d*sqrt(ex))
      do i = 1, nb
         bk(i) = bk1
      end do
      goto 50
   else
         !-----------
         !  x > 1.0
         !-----------
      twox = ex+ex
      blpha = zero
      ratio = zero
      if ( ex <= four ) then
            !------------------------------------------------------------
            !  Calculation of K(alpha+1,x)/K(alpha,x),  1.0 <= x <= 4.0
            !------------------------------------------------------------
         d2 = aint(estm(1)/ex+estm(2))
         m = int(d2)
         d1 = d2+d2
         d2 = d2-half
         d2 = d2*d2
         do i = 2, m
            d1 = d1-two
            d2 = d2-d1
            ratio = (d3+d2)/(twox+d1-ratio)
         end do
            !--------------------------------------------------------------
            !  Calculation of I(|alpha|,x) and I(|alpha|+1,x) by backward
            !    recurrence and K(alpha,x) from the wronskian
            !--------------------------------------------------------------
         d2 = aint(estm(3)*ex+estm(4))
         m = int(d2)
         c = abs(enu)
         d3 = c+c
         d1 = d3-one
         f1 = xmin
         f0 = (two*(c+d2)/ex+half*ex/(c+d2+one))*xmin
         do i = 3, m
            d2 = d2-one
            f2 = (d3+d2+d2)*f0
            blpha = (one+d1/d2)*(f2+blpha)
            f2 = f2/ex+f1
            f1 = f0
            f0 = f2
         end do
         f1 = (d3+two)*f0/ex+f1
         d1 = zero
         t1 = one
         do i = 1, 7
            d1 = c*d1+p(i)
            t1 = c*t1+q(i)
         end do
         p0 = exp(c*(a+c*(p(8)-c*d1/t1)-log(ex)))/ex
         f2 = (c+half-ratio)*f1/ex
         bk1 = p0+(d3*f0-f2+f0+blpha)/(f2+f1+f0)*p0
         bk1 = bk1*exp(-ex)
         wminf = estf(3)*ex+estf(4)
      else
            !----------------------------------------------------------------------
            !  Calculation of K(alpha,x) and K(alpha+1,x)/K(alpha,x), by backward
            !  recurrence, for  x > 4.0
            !----------------------------------------------------------------------
         dm = aint(estm(5)/ex+estm(6))
         m = int(dm)
         d2 = dm-half
         d2 = d2*d2
         d1 = dm+dm
         do i = 2, m
            dm = dm-one
            d1 = d1-two
            d2 = d2-d1
            ratio = (d3+d2)/(twox+d1-ratio)
            blpha = (ratio+ratio*blpha)/dm
         end do
         bk1 = one/((d+d*blpha)*sqrt(ex))
         bk1 = bk1*exp(-ex)
         wminf = estf(5)*(ex-abs(ex-estf(7)))+estf(6)
      end if
         !---------------------------------------------------
         !  Calculation of K(alpha+1,x) from K(alpha,x) and
         !    K(alpha+1,x)/K(alpha,x)
         !---------------------------------------------------
      bk2 = bk1+bk1*(enu+half-ratio)/ex
   end if
      !--------------------------------------------------------------------
      !  Calculation of 'ncalc', K(alpha+i,x), i  =  0, 1, ... , ncalc-1,
      !  K(alpha+i,x)/K(alpha+i-1,x), i  =  ncalc, ncalc+1, ... , nb-1
      !--------------------------------------------------------------------
   ncalc = nb
   bk(1) = bk1
   if ( iend == 0 ) goto 50
   j = 2-k
   if ( j > 0 ) bk(j) = bk2
   if ( iend == 1 ) goto 50
   m = min(int(wminf-enu),iend)
   do i = 2, m
      t1 = bk1
      bk1 = bk2
      twonu = twonu+two
      if ( ex < one ) then
         if ( bk1 >= (xinf/twonu)*ex ) exit
      else
         if ( bk1/ex >= xinf/twonu ) exit
      end if
      bk2 = twonu/ex*bk1+t1
      itemp = i
      j = j+1
      if ( j > 0 ) bk(j) = bk2
   end do
   m = itemp
   if ( m == iend ) goto 50
   ratio = bk2/bk1
   mplus1 = m+1
   ncalc = -1
   do i = mplus1, iend
      twonu = twonu+two
      ratio = twonu/ex+one/ratio
      j = j+1
      if ( j > 1 ) then
         bk(j) = ratio
      else
         if ( bk2 >= xinf/ratio ) goto 50
         bk2 = ratio*bk2
      end if
   end do
   ncalc = max(mplus1-k,1)
   if ( ncalc == 1 ) bk(1) = bk2
   if ( nb == 1 ) goto 50
20 continue
   j = ncalc+1
   do i = j, nb
      if ( bk(ncalc) >= xinf/bk(i) ) goto 50
      bk(i) = bk(ncalc)*bk(i)
      ncalc = i
   end do

50 continue
   ier = ncalc - nb
   if ( ier == 0 ) bessel_k = bk(nb)

end function bessel_k
