function betac( a, b, ier )

!-----------------------------------------------------------------------
!
!     Returns the complete beta function: gamma(a)*gamma(b)/gamma(a+b)
!
!     A     - Input . 1st parameter                           - Real
!                     (A /= 0,-1,-2,... and A <= 2.556E+305)
!     B     - Input . 2nd parameter                           - Real
!                     (B /= 0,-1,-2,... and B <= 2.556E+305)
!                     N.B. Same limitations hold for A+B
!                     Furthermore, if min(A,B,A+B) < 0 then
!                     max(A,B,A+B) < 171.624 is needed
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then betac = 0)
!                     2 = result exceeds limit
!                         (then betac = huge: 1.797...e+308)
!
!     External functions called:
!       GAMMAC  DLGAMA if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  HUGE  MAX  MIN  NINT
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: betac

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: a, b
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: gammac

   real(kind=8), parameter :: zero=0.0_8
   real(kind=8), parameter :: explower=-706.893_8, expupper=-explower
   !     explower = minimum valid argument for the exponential function
   !     expupper = maximum valid argument for the exponential function
   real(kind=8), parameter :: gamupper=171.624_8
   !     gamupper = maximum valid argument for the gamma function
   real(kind=8), parameter :: lgamupper=2.5563e305_8
   !     lgamupper = maximum valid argument for the logamma function

   real(kind=8) :: c, s

   !--------------------------------------------------------------------

   betac = zero
   ier = 0
   c = a + b

      ! Check input
   if ( a == zero .or. a == -abs(nint(a)) .or. a > lgamupper .or.  &
        b == zero .or. b == -abs(nint(b)) .or. b > lgamupper .or.  &
        c == zero .or. c == -abs(nint(c)) .or. c > lgamupper ) then
      ier = 1
      return
   end if

   if ( max(a,b,c) <= gamupper ) then
         ! Use complete Gamma function
      betac = gammac(a) / gammac(c) * gammac(b)
   else
         ! Try LogGamma function
      if ( min(a,b,c) > zero ) then
         s = dlgama(a) + dlgama(b) - dlgama(c)
         if ( s >= explower .and. s <= expupper ) then
            betac = exp(s)
         else if ( s > expupper ) then
            betac = huge(a)
            ier = 2
         end if
      else
         ier = 1
      end if
   end if

end function betac
