function digamma( x )

!-----------------------------------------------------------------------
!
!     Computes the Digamma (Psi) function.
!
!     X     - Input . Value of the variable                   - Real
!
!     Fortran functions called:
!       ABS  COS  HUGE  INT  LOG  MOD  SIN
!
!     From Zhang S. & Jin J. (1996). Computation of Special Functions.
!          Wiley.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: digamma

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x

   !  Local declarations
   !  ------------------

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, ten=10.0_8
   real(kind=8), parameter :: pi=3.141592653589793_8
   real(kind=8), parameter :: el=0.5772156649015329_8     ! Euler constant
   real(kind=8), parameter :: twol2=1.386294361119891_8   ! 2*log(2)
   real(kind=8), parameter :: a1=-0.8333333333333e-1_8,      &
                              a2= 0.83333333333333333e-2_8,  &
                              a3=-0.39682539682539683e-2_8,  &
                              a4= 0.41666666666666667e-2_8,  &
                              a5=-0.75757575757575758e-2_8,  &
                              a6= 0.21092796092796093e-1_8,  &
                              a7=-0.83333333333333333e-1_8,  &
                              a8= 0.4432598039215686_8
   real(kind=8), parameter :: xlimit=1.0e5_8 ! The limit over which
                                             ! integral and half-integral values
                                             ! are not treated specifically

   real(kind=8) :: pix, ps, s, xa, x2
   integer :: k, n

   !--------------------------------------------------------------------

   if ( x <= zero .and. x == int(x) ) then
      digamma = -huge(x)
      return
   end if

   xa = abs(x)
   s = zero

   if ( xa <= xlimit .and. xa == int(xa) ) then   ! |x| = n <= xlimit
      n = xa
      do k = 1, n-1
         s = s + one/k
      end do
      ps = s - el
   else if ( xa <= xlimit .and. xa+half == int(xa+half) ) then   ! |x| = n+1/2 < xlimit
      n = xa - half
      do k = 1, n
         s = s + one/(k+k-1)
      end do
      ps = s + s - el - twol2
   else
      if ( xa < ten ) then   ! When |x| < 10, add an integer to
         n = 10 - int(xa)    ! |x| such that |x| + n > 10
         do k = 0, n-1
            s = s + one/(xa+k)
         end do
         xa = xa + n
      end if
      x2 = one/(xa*xa)
         ! Calculate Psi(|x|+n)
      ps = log(xa) - half/xa + x2*(((((((a8*x2+a7)*x2+a6)*x2+a5)*x2+a4)*x2+a3)*x2+a2)*x2+a1)
         ! Calculate Psi(|x|)
      ps = ps - s
   end if
      ! Calculate Psi(x)
   if ( x < zero ) then
      pix = mod(pi*x,pi)
      ps = ps - pi*cos(pix)/sin(pix) - one/x
   end if
   digamma = ps

end function digamma
