function dgamma( x )

!-----------------------------------------------------------------------
!
!     Returns the complete gamma function
!
!     X     - Input . Argument ( X < XBIG )                   - Real
!
!-----------------------------------------------------------------------
!
! This routine calculates the GAMMA function for a real argument X.
!   Computation is based on an algorithm outlined in reference 1.
!   The program uses rational functions that approximate the GAMMA
!   function to at least 20 significant decimal digits.  Coefficients
!   for the approximation over the interval (1,2) are unpublished.
!   Those for the approximation for X .GE. 12 are from reference 2.
!   The accuracy achieved depends on the arithmetic system, the
!   compiler, the intrinsic functions, and proper selection of the
!   machine-dependent constants.
!
!*******************************************************************
!
! Explanation of machine-dependent constants
!
! beta   - radix for the floating-point representation
! maxexp - the smallest positive power of beta that overflows
! XBIG   - the largest argument for which GAMMA(X) is representable
!          in the machine, i.e., the solution to the equation
!                  GAMMA(XBIG) = beta**maxexp
! XINF   - the largest machine representable floating-point number;
!          approximately beta**maxexp
! EPS    - the smallest positive floating-point number such that
!          1.0+EPS .GT. 1.0
! XMININ - the smallest positive floating-point number such that
!          1/XMININ is machine representable
!
!     Approximate values for some important machines are:
!
!                            beta       maxexp        XBIG
!
! CRAY-1         (S.P.)        2         8191        966.961
! Cyber 180/855
!   under NOS    (S.P.)        2         1070        177.803
! IEEE (IBM/XT,
!   SUN, etc.)   (S.P.)        2          128        35.040
! IEEE (IBM/XT,
!   SUN, etc.)   (D.P.)        2         1024        171.624
! IBM 3033       (D.P.)       16           63        57.574
! VAX D-Format   (D.P.)        2          127        34.844
! VAX G-Format   (D.P.)        2         1023        171.489
!
!                            XINF         EPS        XMININ
!
! CRAY-1         (S.P.)   5.45E+2465   7.11E-15    1.84E-2466
! Cyber 180/855
!   under NOS    (S.P.)   1.26E+322    3.55E-15    3.14E-294
! IEEE (IBM/XT,
!   SUN, etc.)   (S.P.)   3.40E+38     1.19E-7     1.18E-38
! IEEE (IBM/XT,
!   SUN, etc.)   (D.P.)   1.79D+308    2.22D-16    2.23D-308
! IBM 3033       (D.P.)   7.23D+75     2.22D-16    1.39D-76
! VAX D-Format   (D.P.)   1.70D+38     1.39D-17    5.88D-39
! VAX G-Format   (D.P.)   8.98D+307    1.11D-16    1.12D-308
!
!*******************************************************************
!
! Error returns
!
!  The program returns the value XINF for singularities or
!     when overflow would occur.  The computation is believed
!     to be free of underflow and overflow.
!
!  Intrinsic functions required are:
!
!     INT, DBLE, EXP, LOG, REAL, SIN
!
! References: "An Overview of Software Development for Special
!              Functions", W. J. Cody, Lecture Notes in Mathematics,
!              506, Numerical Analysis Dundee, 1975, G. A. Watson
!              (ed.), Springer Verlag, Berlin, 1976.
!
!              Computer Approximations, Hart, Et. Al., Wiley and
!              sons, New York, 1968.
!
!  Latest modification: October 12, 1989
!
!  Authors: W. J. Cody and L. Stoltz
!           Applied Mathematics Division
!           Argonne National Laboratory
!           Argonne, IL 60439
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: dgamma

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x

   !  Local declarations
   !  ------------------

   integer :: i, n
   logical :: parity

      !  Mathematical constants:
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8, twelve=12.0_8
   real(kind=8), parameter :: pi=3.1415926535897932384626434_8
   real(kind=8), parameter :: sqrtpi=0.9189385332046727417803297_8
      !  Machine dependent parameters:
   real(kind=8), parameter :: eps=2.22e-16_8, xbig=171.624_8, xinf=1.79e308_8, xminin=2.23e-308_8

   real(kind=8) :: conv, fact, res, sum, xden, xnum, y, y1, ysq, z

      !  Numerator and denominator coefficients for rational minimax
      !     approximation over (1,2):
   real(kind=8), dimension(8) :: p=(/                                         &
             -1.71618513886549492533811e+0_8, 2.47656508055759199108314e+1_8, &
             -3.79804256470945635097577e+2_8, 6.29331155312818442661052e+2_8, &
              8.66966202790413211295064e+2_8,-3.14512729688483675254357e+4_8, &
             -3.61444134186911729807069e+4_8, 6.64561438202405440627855e+4_8/)
   real(kind=8), dimension(8) :: q=(/                                         &
             -3.08402300119738975254353e+1_8, 3.15350626979604161529144e+2_8, &
             -1.01515636749021914166146e+3_8,-3.10777167157231109440444e+3_8, &
              2.25381184209801510330112e+4_8, 4.75584627752788110767815e+3_8, &
             -1.34659959864969306392456e+5_8,-1.15132259675553483497211e+5_8/)
      !  Coefficients for minimax approximation over (12, INF):
   real(kind=8), dimension(7) :: c=(/                                         &
             -1.910444077728e-03_8,8.4171387781295e-04_8,                     &
             -5.952379913043012e-04_8,7.93650793500350248e-04_8,              &
             -2.777777777777681622553e-03_8,8.333333333333333331554247e-02_8, &
              5.7083835261e-03_8/)

   !--------------------------------------------------------------------

   parity = .false.
   fact = one
   n = 0
   y = x
   if (y <= zero) then
      !------------------------
      !  Argument is negative
      !------------------------
      y = -x
      y1 = aint(y)
      res = y - y1
      if (res /= zero) then
         if (y1 /= aint(y1*half)*two) parity = .true.
         fact = -pi / sin(pi*res)
         y = y + one
      else
         dgamma = xinf
         return
      end if
   end if
      !------------------------
      !  Argument is positive
      !------------------------
   if (y < eps) then
         !------------------
         !  Argument < EPS
         !------------------
      if (y >= xminin) then
            res = one / y
         else
            dgamma = xinf
            return
      end if
   else if (y < twelve) then
      y1 = y
      if (y < one) then
         !------------------------
         !  0.0 < argument < 1.0
         !------------------------
         z = y
         y = y + one
      else
         !-------------------------------------------------------
         !  1.0 < argument < 12.0, reduce argument if necessary
         !-------------------------------------------------------
         n = int(y) - 1
         y = y - dble(n)
         z = y - one
      end if
      !---------------------------------------------------
      !  Evaluate approximation for 1.0 < argument < 2.0
      !---------------------------------------------------
      xnum = zero
      xden = one
      do i = 1, 8
         xnum = (xnum + p(i)) * z
         xden = xden * z + q(i)
      end do
      res = xnum / xden + one
      if (y1 < y) then
         !------------------------------------------------
         !  Adjust result for case  0.0 < argument < 1.0
         !------------------------------------------------
         res = res / y1
      else if (y1 > y) then
         !-------------------------------------------------
         !  Adjust result for case  2.0 < argument < 12.0
         !-------------------------------------------------
         do i = 1, n
            res = res * y
            y = y + one
         end do
      end if
   else
      !---------------------------------
      !  Evaluate for argument >= 12.0
      !---------------------------------
      if (y <= xbig) then
         ysq = y * y
         sum = c(7)
         do i = 1, 6
            sum = sum / ysq + c(i)
         end do
         sum = sum/y - y + sqrtpi
         sum = sum + (y-half)*log(y)
         res = exp(sum)
      else
         dgamma = xinf   ! Error return
         return
      end if
   end if

      !  Final adjustments
   if (parity) res = -res
   if (fact /= one) res = fact / res
   dgamma = res

end function dgamma
