function hg1f1zj( x, a, b, ier )

!-----------------------------------------------------------------------
!
!     Computes the confluent hypergeometric function 1F1.
!
!     X     - Input . Argument                                - Real
!     A     - Input . 1st (numerator) parameter               - Real
!     B     - Input . 2nd (denominator) parameter             - Real
!                     (B /= 0,-1,-2,...)
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then hg1f1 = 0)
!
!     External functions called:
!       GAMMAC
!
!     Fortran functions called:
!       ABS  COS  EXP  INT  NINT
!
!     See
!     Zhang & Jin. (1996). Computation of Special Function. Wiley.
!     Abramowitz M. & Stegun I.A. (1964). Handbook of mathematical
!         mathematical functions.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: hg1f1zj

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, a, b
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: gammac

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8, two=2.0_8
   real(kind=8), parameter :: pi=3.141592653589793_8
   real(kind=8), parameter :: eps=1.0e-15_8
   integer, parameter :: maxiter=500   ! maximum number of iterations

   real(kind=8) :: a0, aa, hg, hg1, hg2, r, r1, r2, rg
   real(kind=8) :: sum1, sum2, ta, tb, tba, xx, xg, y0, y1
   integer :: i, j, k, la, m, n, nl

   !--------------------------------------------------------------------

   hg1f1zj = zero
      ! Check input
   if ( b == zero .or. b == -abs(nint(b)) ) then
      ier = 1
      return
   end if
   ier = 0

   a0 = a
   aa = a
   xx = x
   if ( b == zero .or. b == -abs(int(b)) ) then
      hg1f1zj = 1.0e+307_8
      return
   else if ( aa == zero .or. xx == zero ) then
      hg1f1zj = one
      return
   else if ( aa == -one ) then
      hg1f1zj = one - xx/b
      return
   else if ( aa == b ) then
      hg1f1zj = exp(xx)
      return
   else if ( aa-b == one ) then
      hg1f1zj = (one+xx/b)*exp(xx)
      return
   else if ( aa == one .and. b == two ) then
      hg1f1zj = (exp(xx)-one)/xx
      return
   else if ( aa == int(aa) .and. aa < zero ) then
      m = int(-aa)
      r = one
      hg = one
      do k = 1, m
         r = r*(aa+k-one)/k/(b+k-one)*xx
         hg = hg + r
      end do
      hg1f1zj = hg
      return
   end if

   if ( xx < zero ) then
      aa = b - aa
      a0 = aa
      xx = abs(xx)
   end if
   if ( aa < two) nl = 0
   if ( aa >= two ) then
      nl = 1
      la = int(aa)
      aa = aa - la - one
   end if
   do n = 0, nl
      if ( a0 >= two ) aa = aa + one
      if ( xx <= 30.0_8+abs(b) .or. aa < zero ) then
         hg = one
         rg = one
         do j = 1, maxiter
            rg = rg*(aa+j-one)/(j*(b+j-one))*xx
            hg = hg + rg
            if ( abs(rg/hg) < eps ) exit
         end do
      else
         ta = gammac(aa)
         tb = gammac(b)
         xg = b-aa
         tba = gammac(xg)
         sum1 = one
         sum2 = one
         r1 = one
         r2 = one
         do i = 1, 8
            r1 = -r1*(aa+i-one)*(aa-b+i)/(xx*i)
            r2 = -r2*(b-aa+i-one)*(aa-i)/(xx*i)
            sum1 = sum1 + r1
            sum2 = sum2 + r2
         end do
         hg1 = tb/tba*xx**(-aa)*cos(pi*aa)*sum1
         hg2 = tb/ta*exp(xx)*xx**(aa-b)*sum2
         hg = hg1+hg2
      end if
      if ( n == 0 ) y0 = hg
      if ( n == 1 ) y1 = hg
   end do
   if ( a0 >= two ) then
      do i = 1, la-1
         hg = ((two*aa-b+xx)*y1+(b-aa)*y0)/aa
         y0 = y1
         y1 = hg
         aa = aa + one
      end do
   end if
   if ( x < zero ) hg = hg*exp(x)
   hg1f1zj = hg

end function hg1f1zj
