function hg1f1m3( x, a, b, ier )

!-----------------------------------------------------------------------
!
!     Computes the confluent hypergeometric function 1F1.
!
!     X     - Input . Value of the variable                   - Real
!     A     - Input . 1st (numerator) parameter               - Real
!     B     - Input . 2nd (denominator) parameter             - Real
!                     (B /= 0,-1,-2,...)
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then hg1f1 = 0)
!
!     External functions called:
!       NONE
!
!     Fortran functions called:
!       ABS  EXP  NINT
!
!     See
!     Muller K.E. (2001). Computing the confluent hypergeometric
!         function. Numerische Mathematik, 90, 179-196.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: hg1f1m3

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, a, b
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8, two=2.0_8
   real(kind=8), parameter :: eps=0.223e-15_8, explower=-706.893_8
   integer, parameter :: maxiter=8   ! maximum number of iterations

   !     These constants are machine dependent:
   !     eps   = machine epsilon
   !             (the smallest real such that 1.0 + eps > 1.0)
   !     explower = minimum valid argument for the exponential function

   real(kind=8) :: aa, d, f1, f2, f3, f4, h
   real(kind=8) :: p, p0, p1, p2, q, q0, q1, q2, r, xx
   integer :: i, m

   !--------------------------------------------------------------------

      ! Check input
   if ( b == zero .or. b == -abs(nint(b)) ) then
      hg1f1m3 = 1.0e+300_8
      ier = 1
      return
   end if

   hg1f1m3 = zero
   ier = 0

      ! Special cases
   if ( a == zero .or. x == zero ) then
      hg1f1m3 = one
      return
   else if ( abs((a-b)/a) < eps .and. abs(x) > zero ) then
      hg1f1m3 = exp(x)
      return
   else if ( abs(a+one) < prec ) then
      hg1f1m3 = one - x/b
      return
   else if ( abs(a-b-one) < prec ) then
      hg1f1m3 = (one+x/b)*exp(x)
      return
   else if ( abs(a-one) < prec .and. abs(b-two) < prec ) then
      hg1f1m3 = (exp(x)-one)/x
      return
   else if ( a == nint(a) .and. a < zero ) then
      m = nint(-a)
      r = one
      h = one
      do i = 1, m
         r = r*(i-1+a)/i/(i-1+b)*x
         h = h + r
      end do
      hg1f1m3 = h
      return
   end if

      ! General case: use a rational approximation
   if ( x > zero ) then
      aa = a
      xx = x
   else
      aa = b - aa
      xx = abs(xx)
   end if
   d = b - aa
   x3 = xx*xx*xx
   am3 =
   am2 =
   am1 =
   bm3 =
   bm2 =
   bm1 =
   q0 = one
   q1 = one + xx*(d+one)/(b+b)
   q2 = one + xx*(d+two)/(b+b+two) + xx*xx*(d+one)*(d+two)/(12._8*(b+one))
   p0 = one
   p1 = q1 - xx*d/b
   p2 = q2 -(xx*d/b)*(one+xx*(d+two)/(two*(b+two))) + xx*xx*d*(d+one)/(two*b*(b+one))
   do i = 3, maxitr
      f1 = (i-2-d)/((4*i-6)*(i-1+b))
      f2 = (i+d)*(i-1+d)*(i-2-d)/(4*(i+i-1)*(i+i-3)*(i-2+b)*(i-1+b))
      f3 = (i-2+d)*(i-1+d)*(i-2+d)/(8*((i+i-3)**2)*(i+i-5)*(i-2+b)*(i-1+b))
      f4 = (i-1+d)*(i-1-b)/(2*(i+i-3)*(i-2+b)*(i-1+b))
      p = (one+f1*xx)*am1 + (f4+f2*xx)*am2 + f3*x3*am3
      q = (one+f1*xx)*bm1 + (f4+f2*xx)*bm2 + f3*x3*bm3
      am1 = am2
      am2 = am3
      am3 =
      bm1 = bm2
      bm2 = bm3
      bm3 =
   end do
   hg1f1m3 = p/q
   if ( x > zero ) hg1f1m3 = hg1f1m3*exp(xx)

end function hg1f1m3
