function hg2f1( x, a, b, c, ier )

!-----------------------------------------------------------------------
!
!     Computes the Gauss hypergeometric function 2F1.
!
!     X     - Input . Value of the variable (|X| <= 1)        - Real
!                     (|X| < 1 if c < a+b)
!     A     - Input . 1st (numerator) parameter               - Real
!     B     - Input . 2nd (numerator) parameter               - Real
!     C     - Input . 3rd (denominator) parameter             - Real
!                     (C /= 0,-1,-2,...)
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then hg2f1 = 0)
!                     2 = the hypergeometric series is divergent
!                     3 = maximum number of iterations reached
!                         (then hg2f1 = value at last iteration)
!
!     External functions called:
!       DIGAMMA  GAMMAC  DLGAMA if not in FORTRAN library
!
!     Fortran functions called:
!       ABS  EXP  HUGE  LOG  MAX  MIN  MOD  NINT
!
!     See Abramowitz M. & Stegun I.A. (1964).
!         Handbook of mathematical functions.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: hg2f1

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x, a, b, c
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: digamma, gammac

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8
   real(kind=8), parameter :: epsil=epsilon(one)
   real(kind=8), parameter :: eps=1.0e-14_8
   real(kind=8), parameter :: xlimit=0.95_8
                            ! xlimit: The limit for x over which Psi function
                            ! expansion is used if c-a-b is an integer number
   real(kind=8), parameter :: el=0.5772156649015329_8           ! Euler constant
   real(kind=8), parameter :: sqrpi=1.77245385090551603_8       ! sqrt(pi)
   real(kind=8), parameter :: sqrpilog=0.572364942924700087_8   ! log(sqrt(pi))
   real(kind=8), parameter :: twolog=0.693147180559945309_8     ! log(2)
   real(kind=8), parameter :: explower=-706.893_8, expupper=-explower
   !     explower = minimum valid argument for the exponential function
   !     expupper = maximum valid argument for the exponential function
   real(kind=8), parameter :: gamupper=171.624_8
   !     gamupper = maximum valid argument for the gamma function
   integer, parameter :: limgam=169     ! int(gamupper) - 2

   integer, parameter :: maxiter=5000   ! maximum number of iterations

   real(kind=8) :: aa, bb, cc, d, e, f, pow, s, t, u, v, xl, xx, y
   integer :: i, m
   logical :: aneg, bneg, caneg, cbneg, xneg

   !--------------------------------------------------------------------

   hg2f1 = zero
   ier = 0
   xx = one - x

      ! Check input
   if ( c == zero .or. c == -abs(nint(c)) .or. abs(x) > one .or. &
        xx < eps .and. c-a-b <= zero ) then
      ier = 1
      return
   end if

      ! Special cases
   aneg = a < zero .and. a == nint(a)
   bneg = b < zero .and. b == nint(b)
   caneg = c-a <= zero .and. c-a == nint(c-a)
   cbneg = c-b <= zero .and. c-b == nint(c-b)
   if ( x == zero .or. a == zero .or. b == zero ) then
      hg2f1 = one
      return
   else if ( abs(c-a) <= eps ) then
      hg2f1 = xx**(-b)
      return
   else if ( abs(c-b) <= eps ) then
      hg2f1 = xx**(-a)
      return
   else if ( abs(xx) <= eps .and. c-a-b > zero ) then
      d = max( c, c-a, c-b, c-a-b )
      if ( d <= gamupper ) then
         hg2f1 = gammac(c) / gammac(c-a) * gammac(c-a-b) / gammac(c-b)
      else
         d = min( c, c-a, c-b, c-a-b )
         if ( d > zero ) then
            s = dlgama(c) + dlgama(c-a-b) - dlgama(c-a) - dlgama(c-b)
            if ( s >= explower .and. s <= expupper ) then
               hg2f1 = exp(s)
            else if ( s > expupper ) then
               hg2f1 = huge(one)
               ier = 2
            end if
         else
            hg2f1 = huge(one)
            ier = 2
         end if
      end if
      return
   else if ( abs(one+x) <= eps .and. abs(c-a+b-one) <= eps ) then
      d = max( c, one+a*half-b, half+half*a )
      if ( d <= gamupper ) then
         hg2f1 = gammac(c) / gammac(one+a*half-b) / gammac( half+half*a) * sqrpi / two**a
      else
         d = min( c, one+a*half-b, half+half*a )
         if ( d > zero ) then
            s = (-a)*twolog+sqrpilog + dlgama(c) - dlgama(one+a*half-b) - dlgama(half+half*a)
            if ( s >= explower .and. s <= expupper ) then
               hg2f1 = exp(s)
            else if ( s > expupper ) then
               hg2f1 = huge(one)
               ier = 2
            end if
         else
            hg2f1 = huge(one)
            ier = 2
         end if
      end if
      return
   else if ( aneg .or. bneg ) then
      if ( aneg ) then
         m = abs(nint(a))
         if ( bneg ) m = min(m,abs(nint(b)))
      else
         m = abs(nint(b))
      end if
      s = one
      t = one
      do i = 0, m-1
         t = t * (a+i)/(c+i)*(b+i)/(i+1)*x
         s = s + t
      end do
      hg2f1 = s
      return
   else if ( caneg .or. cbneg ) then
      if ( caneg ) then
         m = abs(nint(c-a))
         if ( cbneg ) m = min(m,abs(nint(c-b)))
      else
         m = abs(nint(c-b))
      end if
      s = one
      t = one
      do i = 0, m-1
         t = t * (c-a+i)/(c+i)*(c-b+i)/(i+1)*x
         s = s + t
      end do
      s = xx**(c-a-b) * s
      hg2f1 = s
      return
   end if

   m = nint(c-a-b)

   if ( x <= xlimit .or. abs(c-a-b-m) > eps .or. abs(m) > limgam  &
        .or. max(c,max(a,b)+max(m,0)) > gamupper ) then
         ! Use power series expansion

      xx = x
      aa = a
      bb = b
      cc = c
      xneg = x < zero
      if ( xneg ) then
         xx = x/(x-one)
         if ( a <= b ) then
            pow = -a
            bb = c - b
         else
            pow = -b
            aa = c - a
         end if
      end if
      s = one
      y = one
      do i = 0, maxiter-1
         y = y * (aa+i)/(cc+i)*(bb+i)*xx/(i+1)
         t = s + y
         if ( s == t ) then
            if ( xneg ) s = (one-x)**pow * s
            hg2f1 = s
            return
         end if
         s = t
      end do
      if ( xneg ) s = (one-x)**pow * s
      hg2f1 = s
      ier = 3

   else
         ! Use Psi function expansion if x exceeds the limit and
         ! m=c-a-b is an integer number

      xl = log(xx)
      d = abs(m)
      if ( m >= 0 ) then
         e = d
         f = zero
         v = one
         if ( m > 0 ) then
            u = xx**d
         else
            u = one
         end if
      else
         e = zero
         f = m
         u = one
         v = xx**f
      end if
      u = gammac(c) / gammac(a+f) / gammac(b+f) * u
      if ( u == zero ) return
      s = (xl+el-digamma(d+one)+digamma(a+e)+digamma(b+e))/gammac(d+one)   ! sum for i=0
      t = (a+e)*(b+e)/gammac(d+one+one)*xx   ! Pochhammer, factorial and power term for i=1
      ier = 3
      do i = 1, maxiter
         y = t * (xl-digamma(i+one)-digamma(d+i+one)+digamma(a+e+i)+digamma(b+e+i))
         if ( abs(y) < epsil*abs(s) ) then
            ier = 0
            exit
         end if
         s = s + y
         t = t * (a+e+i)*(b+e+i)/(i+1)*xx/(d+i+1)
      end do
      if ( mod(m,2) == 0 ) then
         u = -u*s
      else
         u = u*s
      end if
      hg2f1 = u
      if ( m == 0 ) return
      v = gammac(c) / gammac(a+e) * gammac(d) / gammac(b+e) * v
      if ( v == zero ) return
      d = one - d
      s = one
      t = one
      do i = 0, abs(m)-2
         t = t * (a+f+i)*(b+f+i)/(i+1)*xx/(d+i)
         s = s + t
      end do
      hg2f1 = v*s + u

   end if

end function hg2f1
