function hg2f1fm( x, a, b, c, ier )

!-----------------------------------------------------------------------
!!!!!!!!!!!! This the Multiple Precision version of hg2f1    !!!!!!!!!!!
!!!!!!!!!!!! using David Smith's Multiple Precision software !!!!!!!!!!!
!!!!!!!!!!!! alg 8144 TOMS, 27, 4, 377-387 (2001)            !!!!!!!!!!!
!-----------------------------------------------------------------------
!
!     Computes the Gauss hypergeometric function 2F1.
!
!     X     - Input . Value of the variable (|X| <= 1)        - Real
!                     (|X| < 1 if c < a+b)
!     A     - Input . 1st (numerator) parameter               - Real
!     B     - Input . 2nd (numerator) parameter               - Real
!     C     - Input . 3rd (denominator) parameter             - Real
!                     (C /= 0,-1,-2,...)
!     IER   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then hg2f1fm = 0)
!                     2 = the hypergeometric series is divergent
!                     3 = maximum number of iterations reached
!                         (then hg2f1fm = value at last iteration)
!
!     External functions called:
!       FMEULR  FMPI  GAMMA  LOG_GAMMA  PSI  TO_FM
!
!     Fortran functions called:
!       ABS  EPSILON  EXP  HUGE  LOG  MAX  MIN  MOD  NINT
!
!     See Abramowitz M. & Stegun I.A. (1964).
!         Handbook of mathematical functions.
!
!-----------------------------------------------------------------------

   use fmzm  ! Modules for derived-types

   implicit none

   !  Function
   !  --------

   type(fm) :: hg2f1fm

   !  Arguments
   !  ---------

   type(fm), intent(in) :: x, a, b, c
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! type(fm), external :: gamma, log_gamma, psi
   type(fm), external :: fmeulr, fmpi

   type(fm) :: zero, half, one, two
   type(fm) :: epsil
   type(fm) :: eps
   type(fm) :: xlimit
                            ! xlimit: The limit for x over which Psi function
                            ! expansion is used if c-a-b is an integer number
   type(fm) :: el
   type(fm) :: pi
   type(fm) :: sqrpi
   type(fm) :: sqrpilog
   type(fm) :: twolog
   type(fm) :: explower, expupper
   type(fm) :: gamupper

   integer, parameter :: maxiter=9000   ! maximum number of iterations

   type(fm) :: aa, bb, cc, d, e, f, pow, s, t, u, v, xl, xx, y
   type(fm) :: i_fm
   integer :: limgam
   integer :: i, m
   logical :: aneg, bneg, caneg, cbneg, xneg

   !--------------------------------------------------------------------

      ! Specify constants
   zero = to_fm('0.0')
   half = to_fm('0.5')
   one  = to_fm('1.0')
   two  = to_fm('2.0')
   el = fmeulr(el)  ! Euler constant
   pi = fmpi(pi)    ! Pi
   sqrpi = sqrt(pi)
   sqrpilog = log(sqrt(pi))
   twolog = log(two)
   epsil = epsilon(one)
   eps = to_fm('1.0e-14')
   xlimit = to_fm('0.95')   ! The limit for x over which Psi function
                            ! expansion is used if c-a-b is an integer number
   explower = to_fm('-706.893') ! Minimum argument for the exponential function
   expupper = -explower         ! Maximum argument for the exponential function
   gamupper = to_fm('171.624')  ! Maximum argument for the gamma function
   limgam = int(gamupper) - 2

   hg2f1fm = zero
   ier = 0
   xx = one - x

      ! Check input
   if ( c == zero .or. c == -abs(nint(c)) .or. abs(x) > one .or. &
        xx < eps .and. c-a-b <= zero ) then
      ier = 1
      return
   end if

      ! Special cases
   aneg = a < zero .and. a == nint(a)
   bneg = b < zero .and. b == nint(b)
   caneg = c-a <= zero .and. c-a == nint(c-a)
   cbneg = c-b <= zero .and. c-b == nint(c-b)
   if ( x == zero .or. a == zero .or. b == zero ) then
      hg2f1fm = one
      return
   else if ( abs(c-a) <= eps ) then
      hg2f1fm = xx**(-b)
      return
   else if ( abs(c-b) <= eps ) then
      hg2f1fm = xx**(-a)
      return
   else if ( abs(xx) <= eps .and. c-a-b > zero ) then
      d = max( c, c-a, c-b, c-a-b )
      if ( d <= gamupper ) then
         hg2f1fm = gamma(c) / gamma(c-a) * gamma(c-a-b) / gamma(c-b)
      else
         d = min( c, c-a, c-b, c-a-b )
         if ( d > zero ) then
            s = log_gamma(c) + log_gamma(c-a-b) - log_gamma(c-a) - log_gamma(c-b)
            if ( s >= explower .and. s <= expupper ) then
               hg2f1fm = exp(s)
            else if ( s > expupper ) then
               hg2f1fm = huge(one)
               ier = 2
            end if
         else
            hg2f1fm = huge(one)
            ier = 2
         end if
      end if
      return
   else if ( abs(one+x) <= eps .and. abs(c-a+b-one) <= eps ) then
      d = max( c, one+a*half-b, half+half*a )
      if ( d <= gamupper ) then
         hg2f1fm = gamma(c) / gamma(one+a*half-b) / gamma( half+half*a) * sqrpi / two**a
      else
         d = min( c, one+a*half-b, half+half*a )
         if ( d > zero ) then
            s = (-a)*twolog+sqrpilog + log_gamma(c) - log_gamma(one+a*half-b) - log_gamma(half+half*a)
            if ( s >= explower .and. s <= expupper ) then
               hg2f1fm = exp(s)
            else if ( s > expupper ) then
               hg2f1fm = huge(one)
               ier = 2
            end if
         else
            hg2f1fm = huge(one)
            ier = 2
         end if
      end if
      return
   else if ( aneg .or. bneg ) then
      if ( aneg ) then
         m = abs(nint(a))
!!       if ( bneg ) m = min(m,abs(nint(b)))
         if ( bneg ) m = min(abs(nint(a)),abs(nint(b)))
      else
         m = abs(nint(b))
      end if
      s = one
      t = one
      do i = 0, m-1
         i_fm = to_fm(i)
         t = t * (a+i_fm)/(c+i_fm)*(b+i_fm)/(i_fm+one)*x
         s = s + t
      end do
      hg2f1fm = s
      return
   else if ( caneg .or. cbneg ) then
      if ( caneg ) then
         m = abs(nint(c-a))
!!       if ( cbneg ) m = min(m,abs(nint(c-b)))
         if ( cbneg ) m = min(abs(nint(c-a)),abs(nint(c-b)))
      else
         m = abs(nint(c-b))
      end if
      s = one
      t = one
      do i = 0, m-1
         i_fm = to_fm(i)
         t = t * (c-a+i_fm)/(c+i_fm)*(c-b+i_fm)/(i_fm+one)*x
         s = s + t
      end do
      s = xx**(c-a-b) * s
      hg2f1fm = s
      return
   end if

   m = nint(c-a-b)

   if ( x <= xlimit .or. abs(c-a-b-m) > eps .or. abs(m) > limgam  &
        .or. max(c,max(a,b)+max(m,0)) > gamupper ) then
         ! Use power series expansion

      xx = x
      aa = a
      bb = b
      cc = c
      xneg = x < zero
      if ( xneg ) then
         xx = x/(x-one)
         if ( a <= b ) then
            pow = -a
            bb = c - b
         else
            pow = -b
            aa = c - a
         end if
      end if
      s = one
      y = one
      do i = 0, maxiter-1
         y = y * (aa+i)/(cc+i)*(bb+i)*xx/(i+1)
         t = s + y
         if ( s == t ) then
            if ( xneg ) s = (one-x)**pow * s
            hg2f1fm = s
            return
         end if
         s = t
      end do
      if ( xneg ) s = (one-x)**pow * s
      hg2f1fm = s
      ier = 3

   else
         ! Use Psi function expansion if x exceeds the limit and
         ! m=c-a-b is an integer number

      xl = log(xx)
      d = abs(m)
      if ( m >= 0 ) then
         e = d
         f = zero
         v = one
         if ( m > 0 ) then
            u = xx**d
         else
            u = one
         end if
      else
         e = zero
         f = m
         u = one
         v = xx**f
      end if
      u = gamma(c) / gamma(a+f) / gamma(b+f) * u
      if ( u == zero ) return
      s = (xl+el-psi(d+one)+psi(a+e)+psi(b+e))/gamma(d+one)                ! sum for i=0
      t = (a+e)*(b+e)/gamma(d+one+one)*xx    ! Pochhammer, factorial and power term for i=1
      ier = 3
      do i = 1, maxiter
         i_fm = to_fm(i)
         y = t * (xl-psi(i_fm+one)-psi(d+i_fm+one)+psi(a+e+i_fm)+psi(b+e+i_fm))
         if ( abs(y) < epsil*abs(s) ) then
            ier = 0
            exit
         end if
         s = s + y
         t = t * (a+e+i_fm)*(b+e+i_fm)/(i_fm+one)*xx/(d+i_fm+one)
      end do
      if ( mod(m,2) == 0 ) then
         u = -u*s
      else
         u = u*s
      end if
      hg2f1fm = u
      if ( m == 0 ) return
      v = gamma(c) / gamma(a+e) * gamma(d) / gamma(b+e) * v
      if ( v == zero ) return
      d = one - d
      s = one
      t = one
      do i = 0, abs(m)-2
         i_fm = to_fm(i)
         t = t * (a+f+i_fm)*(b+f+i_fm)/(i_fm+one)*xx/(d+i_fm)
         s = s + t
      end do
      hg2f1fm = v*s + u

   end if

end function hg2f1fm
