function polygam( m, x, ierr )

!-----------------------------------------------------------------------
!
!     Computes the Polygamma function of order m, i.e.
!     the mth derivative of the digamma (psi) function, i.e.
!     the (m+1)th derivative of the logarithm of the gamma function.
!
!     M     - Input . Order of the function       (0<=M<=100) - Integer
!     X     - Input . Value of the variable             (X>0) - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then fcdf = 0.)
!                     2 = fatal error (underflow or overflow)
!                     3 = auxiliary array in dpsifn not large
!                         enough
!
!     Subroutines called:
!        DPSIFN
!     Fortran functions called:
!        GAMMA  MOD
!
!     Uses Algorithm 610, collected algorithms from ACM.
!     Algorithm appeared in ACM-Trans. Math. Software, vol.9, no. 4,
!     dec., 1983, p. 494-502.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: polygam

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: x
   integer, intent(in) :: m
   integer, intent(out) :: ierr

   !  Local declarations
   !  ------------------

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   real(kind=8) :: w(1)
   integer :: kode, n

   !--------------------------------------------------------------------

   if ( x <= zero .or. m < 0 .or. m > 100 ) then
      ierr = 1
      polygam = zero
      return
   end if
   call dpsifn( x, m, 1, 1, w, ierr )
   if ( ierr == 0 ) then
      polygam = w(1)*gamma(m+one)
      if ( mod(m,2) == 0 ) polygam = -polygam
   else
      polygam = zero
   end if

end function polygam

subroutine dpsifn(x, n, kode, m, ans, iflag)

!     WRITTEN BY D.E. AMOS, JUNE, 1982.
!
!     REFERENCES
!         HANDBOOK OF MATHEMATICAL FUNCTIONS, AMS 55, NATIONAL BUREAU
!         OF STANDARDS BY M. ABRAMOWITZ AND I.A. STEGUN, 1964, PP.
!         258-260, EQTNS. 6.3.5, 6.3.18, 6.4.6, 6.4.9, 6.4.10
!
!         ACM TRANS. MATH SOFTWARE, 1983.
!
!     ABSTRACT     *** A DOUBLE PRECISION ROUTINE ***
!         DPSIFN COMPUTES M MEMBER SEQUENCES OF SCALED DERIVATIVES OF
!         THE PSI FUNCTION
!
!              W(K,X)=(-1)**(K+1)*PSI(K,X)/GAMMA(K+1)
!
!         K=N,...,N+M-1 WHERE PSI(K,X) IS THE K-TH DERIVATIVE OF THE PSI
!         FUNCTION.  ON KODE=1, DPSIFN RETURNS THE SCALED DERIVATIVES
!         AS DESCRIBED.  KODE=2 IS OPERATIVE ONLY WHEN K=0 AND DPSIFN
!         RETURNS -PSI(X) + LN(X).  THAT IS, THE LOGARITHMIC BEHAVIOR
!         FOR LARGE X IS REMOVED WHEN KODE=2 AND K=0. WHEN SUMS OR
!         DIFFERENCES OF PSI FUNCTIONS ARE COMPUTED, THE LOGARITHMIC
!         TERMS CAN BE COMBINED ANALYTICALLY AND COMPUTED SEFARATELY
!         TO HELP RETAIN SIGNIFICANT DIGITS.
!
!         THE BASIC METHOD OF EVALUATION IS THE ASYMPTOTIC EXPANSION
!         FOR LARGE X.GE.XMIN FOLLOWED BY BACKWARD RECURSION ON A TWO
!         TERM RECURSION RELATION
!
!                  W(X+1) + X**(-N-1) = W(X).
!
!         THIS IS SUPPLEMENTED BY A SERIES
!
!                  SUM( (X+K)**(-N-1) , K=0,1,2,... )
!
!         WHICH CONVERGES RAPIDLY FOR LARGE N. BOTH XMIN AND THE
!         NUMBER OF TERMS OF THE SERIES ARE CALCULATED FROM THE UNIT
!         ROUND OFF OF THE MACHINE ENVIRONMENT.
!
!         FUNCTIONS I1MACH AND D1MACH MUST BE INITIALIZED ACCORDING
!         TO  PROLOGUE INSTRUCTIONS. FIFTEEN MACHINE ENVIRONMENTS
!         CAN BE DEFINED BY MAKING APPROPRIATE COMMENT CARDS ACTIVE
!         IN I1MACH AND D1MACH.
!
!         THE NOMINAL COMPUTATIONAL ACCURACY IS THE MAXIMUM OF UNIT
!         ROUNDOFF (=D1MACH(4)) AND 1.0D-18 SINCE CRITICAL CONSTANTS
!         ARE GIVEN TO ONLY 18 DIGITS.
!
!         DPSIFN CALLS I1MACH, D1MACH, XERROR PACKAGE(18 ROUTINES)
!
!         PSIFN IS THE SINGLE PRECISION VERSION OF DPSIFN.
!
!     DESCRIPTION OF ARGUMENTS
!
!         INPUT      X IS DOUBLE PRECISION
!           X      - ARGUMENT, X.GT.0.0D0
!           N      - FIRST MEMBER OF THE SEQUENCE, 0.LE.N.LE.100
!                    N=0 GIVES ANS(1) = -PSI(X)       ON KODE=1
!                                       -PSI(X)+LN(X) ON KODE=2
!           KODE   - SELECTION PARAMETER
!                    KODE=1 RETURNS SCALED DERIVATIVES OF THE PSI
!                    FUNCTION
!                    KODE=2 RETURNS SCALED DERIVATIVES OF THE PSI
!                    FUNCTION EXCEPT WHEN N=0. IN THIS CASE,
!                    ANS(1) = -PSI(X) + LN(X) IS RETURNED.
!           M      - NUMBER OF MEMBERS OF THE SEQUENCE, M.GE.1
!
!         OUTPUT     ANS IS DOUBLE PRECISION
!           ANS    - A VECTOR OF LENGTH AT LEAST M WHOSE FIRST M
!                    COMPONENTS CONTAIN THE SEQUENCE OF  DERIVATIVES
!                    SCALED ACCORDING TO KODE
!
!           iflag  - error condition
!         ERROR CONDITIONS
!           AN IMPROPER INPUT IS A FATAL ERROR
!           AN OVERFLOW IS A FATAL ERROR
!***END PROLOGUE

   double precision, intent(in) :: x
   integer, intent(in) :: n, kode, m
   double precision, intent(out) :: ans(m)
   integer, intent(out) :: iflag

!! double precision, external :: d1mach
!! integer, external :: i1mach

   integer, parameter :: nmax=100
   integer :: i, j, k, kontr, mx, nn, np, nx
   double precision, parameter :: zero=0.0d0, half=0.5d0, one=1.0d0
   double precision :: arg, den, elim, eps, fln, fn, fnp, fns,      &
      fx, rln, rxsq, r1m4, r1m5, s, slope, t, ta, tk, tol, tols,    &
      tss, tst, tt, t1, t2, wdtol, xdmln, xdmy, xinc, xln,          &
      xm, xmin, xq, yint
   double precision :: trm(22), trmr(nmax)
!-----------------------------------------------------------------------
!             BERNOULLI NUMBERS
!-----------------------------------------------------------------------
   double precision :: b(22)=(/1.00000000000000000d+00, &
    -5.00000000000000000d-01,1.66666666666666667d-01,   &
    -3.33333333333333333d-02,2.38095238095238095d-02,   &
    -3.33333333333333333d-02,7.57575757575757576d-02,   &
    -2.53113553113553114d-01,1.16666666666666667d+00,   &
    -7.09215686274509804d+00,5.49711779448621554d+01,   &
    -5.29124242424242424d+02,6.19212318840579710d+03,   &
    -8.65802531135531136d+04,1.42551716666666667d+06,   &
    -2.72982310678160920d+07,6.01580873900642368d+08,   &
    -1.51163157670921569d+10,4.29614643061166667d+11,   &
    -1.37116552050883328d+13,4.88332318973593167d+14,   &
    -1.92965793419400681d+16/)

   iflag = 1
   if (x.le.zero .or. n.lt.0 .or. kode.lt.1 .or. kode.gt.2 .or. m.lt.1) return
   iflag = 0
   nn = n + m - 1
   fn = dble(float(nn))
   fnp = fn + one
!! nx = min0(-i1mach(15),i1mach(16))
   nx = min0(-minexponent(zero),maxexponent(zero))
!! r1m5 = d1mach(5)
   r1m5 = log10(dble(radix(zero)))
!! r1m4 = d1mach(4)*half
   r1m4 = epsilon(zero)*half
   wdtol = dmax1(r1m4,0.5d-18)
!-----------------------------------------------------------------------
!     ELIM = APPROXIMATE EXPONENTIAL OVER AND UNDERFLOW LIMIT
!-----------------------------------------------------------------------
   elim = 2.302d0*(dble(float(nx))*r1m5-3.0d0)
   xln = dlog(x)
   t = fnp*xln
!-----------------------------------------------------------------------
!     OVERFLOW AND UNDERFLOW TEST FOR SMALL AND LARGE X
!-----------------------------------------------------------------------
   if (dabs(t).gt.elim) then
      iflag = 2
      return
   end if
   if (x.lt.wdtol) go to 40
!-----------------------------------------------------------------------
!     COMPUTE XMIN AND THE NUMBER OF TERMS OF THE SERIES, FLN+1
!-----------------------------------------------------------------------
!! rln = r1m5*dble(float(i1mach(14)))
   rln = r1m5*dble(float(digits(zero)))
   rln = dmin1(rln,18.06d0)
   fln = dmax1(rln,3.0d0) - 3.0d0
   yint = 3.50d0 + 0.40d0*fln
   slope = 0.21d0 + fln*(0.0006038d0*fln+0.008677d0)
   xm = yint + slope*fn
   mx = int(sngl(xm)) + 1
   xmin = dble(float(mx))
   if (n > 0) then
      xm = -2.302d0*rln - dmin1(zero,xln)
      fns = dble(float(n))
      arg = xm/fns
      arg = dmin1(zero,arg)
      eps = dexp(arg)
      xm = one - eps
      if (dabs(arg).lt.1.0d-3) xm = -arg
      fln = x*xm/eps
      xm = xmin - x
      if (xm.gt.7.0d0 .and. fln.lt.15.0d0) go to 30
   end if
   xdmy = x
   xdmln = xln
   xinc = zero
   if (x < xmin) then
      nx = int(sngl(x))
      xinc = xmin - dble(float(nx))
      xdmy = x + xinc
      xdmln = dlog(xdmy)
   end if
!-----------------------------------------------------------------------
!     GENERATE W(N+M-1,X) BY THE ASYMPTOTIC EXPANSION
!-----------------------------------------------------------------------
   t = fn*xdmln
   t1 = xdmln + xdmln
   t2 = t + xdmln
   tk = dmax1(dabs(t),dabs(t1),dabs(t2))
   if (tk.gt.elim) then
      iflag = 2
      return
   end if
   tss = dexp(-t)
   tt = half/xdmy
   t1 = tt
   tst = wdtol*tt
   if (nn.ne.0) t1 = tt + one/fn
   rxsq = one/(xdmy*xdmy)
   ta = half*rxsq
   t = fnp*ta
   s = t*b(3)
   if (dabs(s) >= tst) then
      tk = 2.0d0
      do k=4,22
        t = t*((tk+fn+one)/(tk+one))*((tk+fn)/(tk+2.0d0))*rxsq
        trm(k) = t*b(k)
        if (dabs(trm(k)).lt.tst) exit
        s = s + trm(k)
        tk = tk + 2.0d0
      end do
   end if
   s = (s+t1)*tss
   if (xinc /= zero) then
!-----------------------------------------------------------------------
!     BACKWARD RECUR FROM XDMY TO X
!-----------------------------------------------------------------------
      nx = int(sngl(xinc))
      np = nn + 1
      if (nx.gt.nmax) then
         iflag = 3
         return
      end if
      if (nn.eq.0) go to 10
      xm = xinc - one
      fx = x + xm
!-----------------------------------------------------------------------
!     THIS LOOP SHOULD NOT BE CHANGED. FX IS ACCURATE WHEN X IS SMALL
!-----------------------------------------------------------------------
      do i=1,nx
        trmr(i) = fx**(-np)
        s = s + trmr(i)
        xm = xm - one
        fx = x + xm
      end do
   end if
   ans(m) = s
   if (fn.eq.zero) go to 20
!-----------------------------------------------------------------------
!     GENERATE LOWER DERIVATIVES, J.LT.N+M-1
!-----------------------------------------------------------------------
   if (m.eq.1) return
   do j=2,m
     fnp = fn
     fn = fn - one
     tss = tss*xdmy
     t1 = tt
     if (fn.ne.zero) t1 = tt + one/fn
     t = fnp*ta
     s = t*b(3)
     if (dabs(s) >= tst) then
        tk = 3.0d0 + fnp
        do k=4,22
          trm(k) = trm(k)*fnp/tk
          if (dabs(trm(k)).lt.tst) exit
          s = s + trm(k)
          tk = tk + 2.0d0
        end do
     end if
     s = (s+t1)*tss
     if (xinc /= zero) then
        if (fn.eq.zero) go to 10
        xm = xinc - one
        fx = x + xm
        do i=1,nx
          trmr(i) = trmr(i)*fx
          s = s + trmr(i)
          xm = xm - one
          fx = x + xm
        end do
     end if
     mx = m - j + 1
     ans(mx) = s
     if (fn.eq.zero) go to 20
   end do
   return
!-----------------------------------------------------------------------
!     RECURSION FOR N = 0
!-----------------------------------------------------------------------
10 continue
   do i=1,nx
     s = s + one/(x+dble(float(nx-i)))
   end do
20 continue
   if (kode == 1) then
      ans(1) = s - xdmln
      return
   end if
   if (xdmy.eq.x) return
   xq = xdmy/x
   ans(1) = s - dlog(xq)
   return
!-----------------------------------------------------------------------
!     COMPUTE BY SERIES (X+K)**(-(N+1)) , K=0,1,2,...
!-----------------------------------------------------------------------
30 continue
   nn = int(sngl(fln)) + 1
   np = n + 1
   t1 = (fns+one)*xln
   t = dexp(-t1)
   s = t
   den = x
   do i=1,nn
     den = den + one
     trm(i) = den**(-np)
     s = s + trm(i)
   end do
   ans(1) = s
   if (n == 0) then
      if (kode.eq.2) ans(1) = s + xln
   end if
   if (m.eq.1) return
!-----------------------------------------------------------------------
!     GENERATE HIGHER DERIVATIVES, J.GT.N
!-----------------------------------------------------------------------
   tol = wdtol/5.0d0
   do j=2,m
     t = t/x
     s = t
     tols = t*tol
     den = x
     do i=1,nn
       den = den + one
       trm(i) = trm(i)/den
       s = s + trm(i)
       if (trm(i).lt.tols) exit
     end do
     ans(j) = s
   end do
   return
!-----------------------------------------------------------------------
!     SMALL X.LT.UNIT ROUND OFF
!-----------------------------------------------------------------------
40 continue
   ans(1) = x**(-n-1)
   if (m > 1) then
      k = 1
      do i=2,m
        ans(k+1) = ans(k)/x
        k = k + 1
      end do
   end if
   if (n.ne.0) return
   if (kode.eq.2) ans(1) = ans(1) + xln

end subroutine dpsifn
