function gamdvr( alpha, idum, ier )

!-----------------------------------------------------------------------
!
!     Returns a random variable distributed according to a Gamma
!     distribution with parameter aplha.
!
!     alpha - Input . Parameter of the Gamma distribution     - Real
!                     (epsilon < alpha < 1/epsilon )
!     idum  - Input . A negative integer to initialize the    - Integer
!             Output. random sequence. Thereafter do not
!                     alter this argument between successive
!                     deviates in a sequence
!     ier   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument alpha
!                         (then gamdvr = -1.0)
!                     2 = underflow
!
!     External functions called:
!        GAMDVI  RAN12
!     Fortran  functions called:
!        EPSILON  HUGE  LOG  MOD  NINT
!
!-----------------------------------------------------------------------
!
!     REMARK :
!
!        For integral values of alpha, the function gamdev is used
!        (Press et al. (1992), Numerical Recipes in Fortran, Cambridge
!        University Press).
!        For non-integral values of alpha, the ratio method is used
!        when alpha > 1, and the acceptance-rejection method is used
!        when alpha < 1 (see K. Lange (1999), Numerical Analysis for
!        Statisticians, Springer).
!
!-----------------------------------------------------------------------

   implicit none

!     Function
!     --------
   real(kind=8) :: gamdvr

!     Arguments
!     ---------
   real(kind=8), intent(in) :: alpha
   integer, intent(inout) :: idum
   integer, intent(out) :: ier

!     Local declarations
!     ------------------
   real(kind=8), external :: gamdvi, ran12

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8
   real(kind=8), parameter :: e=2.7182818284590452_8
   real(kind=8) :: am1, ap1, r, u, uk, v, vk, w, x

   !--------------------------------------------------------------------

   ier = 0
   !  Test for valid input arguments
   if ( alpha < epsilon(alpha) .or. one < alpha*epsilon(alpha) ) then
      ier = 1
      gamdvr = -one
      return
   end if

   !  If alpha is an integral value, use Gamdev algorithm
   if ( mod(alpha,one) <= epsilon(alpha) .and. alpha <= huge(1) ) then
      gamdvr = gamdvi( nint(alpha), idum, ier )
      return
   end if

   if ( alpha > one ) then
      !  Use ratio method (see K. Lange (1999), p277)

      ap1 = alpha + one
      am1 = one/(alpha - one)
      r = ((ap1*am1)**(half*alpha))*sqrt(ap1/am1)/e
      do
         u = ran12( idum )
         v = ran12( idum )
         x = r*v/u
         w = x * am1
         if ( two*am1*log(u)-one-log(w)+w <= zero ) exit
      end do

   else
      !  use acceptance-rejection method (see k. lange (1999), p283)

      am1 = alpha - one
      vk = one/alpha
      uk = e/(e+alpha)
      do
         u = ran12( idum )
         if ( u < uk ) then
            x = ran12(idum)**vk
            if ( x <= zero ) then
               ier = 2
               gamdvr = zero
               return
            end if
            w = exp(-x)
         else
            x = one - log(ran12(idum))
            w = x**am1
         end if
         v = ran12( idum )
         if ( v <= w ) exit
      end do

   end if

   gamdvr = x

end function gamdvr
