function bindev( n, prob, idum, ier )

!-----------------------------------------------------------------------
!
!     Returns an integer random variable distributed according to a
!     binomial distribution with parameters n and prob.
!
!     n     - Input . Parameter of the binomial distribution  - Integer
!                     The number of trials in the binomial
!                     distribution (0.0 < n)
!     prob  - Input . Parameter of the binomial distribution  - Real
!                     The probability of an event in each
!                     trial of the binomial distribution
!                     (0.0 < prob < 1.0)
!     idum  - Input . A negative integer to initialize the    - Integer
!             Output. random sequence. Thereafter do not
!                     alter this argument between successive
!                     deviates in a sequence
!     ier   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument
!                         (then bindev = -1)
!
!     External functions called:
!        RAN1
!     Fortran  functions called:
!        ABS  LOG  MIN  ABS  INT  SQRT
!
!-----------------------------------------------------------------------
!
!     INTEGER FUNCTION IGNBIN( N, P )
!
!                    GENerate BINomial random deviate
!
!
!                              Function
!
!
!     Generates a single random deviate from a binomial
!     distribution whose number of trials is N and whose
!     probability of an event in each trial is P.
!
!
!                              Arguments
!
!
!     N  --> The number of trials in the binomial distribution
!            from which a random deviate is to be generated.
!                              INTEGER N
!
!     P  --> The probability of an event in each trial of the
!            binomial distribution from which a random deviate
!            is to be generated.
!                              REAL P
!
!     IGNBIN <-- A random deviate yielding the number of events
!                from N independent trials, each of which has
!                a probability of event P.
!                              INTEGER IGNBIN
!
!
!                              Note
!
!
!     Uses RANF so the value of the seeds, ISEED1 and ISEED2 must be set
!     by a call similar to the following
!          DUM = RANSET( ISEED1, ISEED2 )
!
!
!                              Method
!
!
!     This is algorithm BTPE from:
!
!         Kachitvichyanukul, V. and Schmeiser, B. W.
!
!         Binomial Random Variate Generation.
!         Communications of the ACM, 31, 2
!         (February, 1988) 216.
!
!**********************************************************************
!     SUBROUTINE BTPEC(N,PP,ISEED,JX)
!
!     BINOMIAL RANDOM VARIATE GENERATOR
!     MEAN .LT. 30 -- INVERSE CDF
!       MEAN .GE. 30 -- ALGORITHM BTPE:  ACCEPTANCE-REJECTION VIA
!       FOUR REGION COMPOSITION.  THE FOUR REGIONS ARE A TRIANGLE
!       (SYMMETRIC IN THE CENTER), A PAIR OF PARALLELOGRAMS (ABOVE
!       THE TRIANGLE), AND EXPONENTIAL LEFT AND RIGHT TAILS.
!
!     BTPE REFERS TO BINOMIAL-TRIANGLE-PARALLELOGRAM-EXPONENTIAL.
!     BTPEC REFERS TO BTPE AND "COMBINED."  THUS BTPE IS THE
!       RESEARCH AND BTPEC IS THE IMPLEMENTATION OF A COMPLETE
!       USABLE ALGORITHM.
!     REFERENCE:  VORATAS KACHITVICHYANUKUL AND BRUCE SCHMEISER,
!       "BINOMIAL RANDOM VARIATE GENERATION,"
!       COMMUNICATIONS OF THE ACM, FORTHCOMING
!     WRITTEN:  SEPTEMBER 1980.
!       LAST REVISED:  MAY 1985, JULY 1987
!     REQUIRED SUBPROGRAM:  RAND() -- A UNIFORM (0,1) RANDOM NUMBER
!                           GENERATOR
!     ARGUMENTS
!
!       N : NUMBER OF BERNOULLI TRIALS            (INPUT)
!       PP : PROBABILITY OF SUCCESS IN EACH TRIAL (INPUT)
!       ISEED:  RANDOM NUMBER SEED                (INPUT AND OUTPUT)
!       JX:  RANDOMLY GENERATED OBSERVATION       (OUTPUT)
!
!     VARIABLES
!       PSAVE: VALUE OF PP FROM THE LAST CALL TO BTPEC
!       NSAVE: VALUE OF N FROM THE LAST CALL TO BTPEC
!       XNP:  VALUE OF THE MEAN FROM THE LAST CALL TO BTPEC
!
!       P: PROBABILITY USED IN THE GENERATION PHASE OF BTPEC
!       FFM: TEMPORARY VARIABLE EQUAL TO XNP + P
!       M:  INTEGER VALUE OF THE CURRENT MODE
!       FM:  FLOATING POINT VALUE OF THE CURRENT MODE
!       XNPQ: TEMPORARY VARIABLE USED IN SETUP AND SQUEEZING STEPS
!       P1:  AREA OF THE TRIANGLE
!       C:  HEIGHT OF THE PARALLELOGRAMS
!       XM:  CENTER OF THE TRIANGLE
!       XL:  LEFT END OF THE TRIANGLE
!       XR:  RIGHT END OF THE TRIANGLE
!       AL:  TEMPORARY VARIABLE
!       XLL:  RATE FOR THE LEFT EXPONENTIAL TAIL
!       XLR:  RATE FOR THE RIGHT EXPONENTIAL TAIL
!       P2:  AREA OF THE PARALLELOGRAMS
!       P3:  AREA OF THE LEFT EXPONENTIAL TAIL
!       P4:  AREA OF THE RIGHT EXPONENTIAL TAIL
!       U:  A U(0,P4) RANDOM VARIATE USED FIRST TO SELECT ONE OF THE
!           FOUR REGIONS AND THEN CONDITIONALLY TO GENERATE A VALUE
!           FROM THE REGION
!       V:  A U(0,1) RANDOM NUMBER USED TO GENERATE THE RANDOM VALUE
!           (REGION 1) OR TRANSFORMED INTO THE VARIATE TO ACCEPT OR
!           REJECT THE CANDIDATE VALUE
!       IX:  INTEGER CANDIDATE VALUE
!       X:  PRELIMINARY CONTINUOUS CANDIDATE VALUE IN REGION 2 LOGIC
!           AND A FLOATING POINT IX IN THE ACCEPT/REJECT LOGIC
!       K:  ABSOLUTE VALUE OF (IX-M)
!       F:  THE HEIGHT OF THE SCALED DENSITY FUNCTION USED IN THE
!           ACCEPT/REJECT DECISION WHEN BOTH M AND IX ARE SMALL
!           ALSO USED IN THE INVERSE TRANSFORMATION
!       R: THE RATIO P/Q
!       G: CONSTANT USED IN CALCULATION OF PROBABILITY
!       MP:  MODE PLUS ONE, THE LOWER INDEX FOR EXPLICIT CALCULATION
!            OF F WHEN IX IS GREATER THAN M
!       IX1:  CANDIDATE VALUE PLUS ONE, THE LOWER INDEX FOR EXPLICIT
!             CALCULATION OF F WHEN IX IS LESS THAN M
!       I:  INDEX FOR EXPLICIT CALCULATION OF F FOR BTPE
!       AMAXP: MAXIMUM ERROR OF THE LOGARITHM OF NORMAL BOUND
!       YNORM: LOGARITHM OF NORMAL BOUND
!       ALV:  NATURAL LOGARITHM OF THE ACCEPT/REJECT VARIATE V
!
!       X1,F1,Z,W,Z2,X2,F2, AND W2 ARE TEMPORARY VARIABLES TO BE
!       USED IN THE FINAL ACCEPT/REJECT TEST
!
!       QN: PROBABILITY OF NO SUCCESS IN N TRIALS
!
!     REMARK
!       IX AND JX COULD LOGICALLY BE THE SAME VARIABLE, WHICH WOULD
!       SAVE A MEMORY POSITION AND A LINE OF CODE.  HOWEVER, SOME
!       COMPILERS (E.G.,CDC MNF) OPTIMIZE BETTER WHEN THE ARGUMENTS
!       ARE NOT INVOLVED.
!
!     ISEED NEEDS TO BE DOUBLE PRECISION IF THE IMSL ROUTINE
!     GGUBFS IS USED TO GENERATE UNIFORM RANDOM NUMBER, OTHERWISE
!     TYPE OF ISEED SHOULD BE DICTATED BY THE UNIFORM GENERATOR
!
!-----------------------------------------------------------------------
!
!     March, 2003: changed to Fortran90 and
!     ranf changed to ran1
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   integer :: bindev

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: prob
   integer, intent(in) :: n
   integer, intent(inout) :: idum
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real, external :: ran1
   integer, save :: nsave=-1
   integer :: i, ix, ix1, k, m, mp
   real, save :: psave=-1.0
   real :: al, alv, amaxp, c, f, f1, f2, ffm, fm, g
   real :: p, pp, p1, p2, p3, p4, q, qn, r, u, v, w, w2
   real :: x, x1, x2, xl, xll, xlr, xm, xnp, xnpq, xr, ynorm, z, z2

   !-----------------------------------------------------------------

   pp = prob
   if ( n <= 0 .or. pp <= 0.0 .or. pp >= 1.0 ) then
      ier = 1
      bindev = -1
      return
   end if
   ier = 0

   !**** Determine appropriate algorithm and whether setup is necessary

   if ( pp /= psave .or. n /= nsave ) then

      !**** Setup, perform only when parameters change

      psave = pp
      p = min(psave,1.-psave)
      q = 1. - p
      xnp = n*p
      nsave = n
      if ( xnp < 30. ) then   !  Inverse cdf logic for mean less than 30
         qn = q**n
         r = p/q
         g = r*(n+1)
      else
         ffm = xnp + p
         m = ffm
         fm = m
         xnpq = xnp*q
         p1 = int(2.195*sqrt(xnpq)-4.6*q) + 0.5
         xm = fm + 0.5
         xl = xm - p1
         xr = xm + p1
         c = 0.134 + 20.5/ (15.3+fm)
         al = (ffm-xl)/ (ffm-xl*p)
         xll = al* (1.+.5*al)
         al = (xr-ffm)/ (xr*q)
         xlr = al* (1.+.5*al)
         p2 = p1* (1.+c+c)
         p3 = p2 + c/xll
         p4 = p3 + c/xlr
      end if
   end if

   if ( xnp >= 30. ) then

      !**** Generate variate

      do
         u = ran1(idum)*p4
         v = ran1(idum)

         if ( u <= p1 ) then        !  Triangular region
            ix = xm - p1*v + u
            goto 10
         else if ( u <= p2 ) then   !  Parallelogram region
            x = xl + (u-p1)/c
            v = v*c + 1. - abs(xm-x)/p1
            if ( v > 1. .or. v <= 0. ) cycle
            ix = x
         else if ( u <= p3 ) then   !  Left tail
            ix = xl + log(v)/xll
            if ( ix < 0 ) cycle
            v = v* (u-p2)*xll
         else                       !  Right tail
            ix = xr - log(v)/xlr
            if ( ix > n ) cycle
            v = v* (u-p3)*xlr
         end if

         !**** Determine appropriate way to perform accept/reject test

         k = abs(ix-m)
         if ( k > 20 .and. k < xnpq/2-1 ) then

            !  Squeezing using upper and lower bounds on log(f(x))
            amaxp = (k/xnpq)* ((k* (k/3.+.625)+.1666666666666)/xnpq+.5)
            ynorm = -k*k/ (2.*xnpq)
            alv = log(v)
            if ( alv < ynorm-amaxp ) goto 10
            if ( alv > ynorm+amaxp ) cycle

            !  Stirling's formula to machine accuracy for
            !  the final acceptance/rejection test
            x1 = ix + 1
            f1 = fm + 1.
            z = n + 1 - fm
            w = n - ix + 1.
            z2 = z*z
            x2 = x1*x1
            f2 = f1*f1
            w2 = w*w
            if ( alv > (xm*log(f1/x1)+ (n-m+.5)*log(z/w)+                    &
                       (ix-m)*log(w*p/ (x1*q))+ (13860.- (462.- (132.- (99.- &
               140./f2)/f2)/f2)/f2)/f1/166320.+ (13860.- (462.- (132.- (99.- &
               140./z2)/z2)/z2)/z2)/z /166320.+ (13860.- (462.- (132.- (99.- &
               140./x2)/x2)/x2)/x2)/x1/166320.+ (13860.- (462.- (132.- (99.- &
               140./w2)/w2)/w2)/w2)/w /166320.) ) cycle
            goto 10
         end if

         !  Explicit evaluation

         f = 1.0
         r = p/q
         g = (n+1)*r
         if ( m < ix ) then
            mp = m + 1
            do i = mp,ix
               f = f* (g/i-r)
            end do
         else if ( m > ix ) then
            ix1 = ix + 1
            do i = ix1,m
               f = f/ (g/i-r)
            end do
         end if
         if ( v <= f ) goto 10
      end do

   else

      do
         ix = 0
         f = qn
         u = ran1(idum)
         if ( u < f ) goto 10
         do ix = 1, 111
            u = u - f
            f = f*(g/ix-r)
            if ( u < f ) goto 10
         end do
      end do

   end if

   10 continue
   if ( psave > 0.5 ) ix = n - ix
   bindev = ix

end function bindev
