function genbet( aa, bb, idum, ier )

!-----------------------------------------------------------------------
!
!     Returns a random variable distributed according to a Beta
!     distribution with parameters aa and bb.
!
!     aa    - Input . 1st parameter of the Beta distributio   - Real
!                     (1e-37 < aa )
!     bb    - Input . 2nd parameter of the Beta distribution  - Real
!                     (1e-37 < bb )
!     idum  - Input . A negative integer to initialize the    - Integer
!             Output. random sequence. Thereafter do not
!                     alter this argument between successive
!                     deviates in a sequence
!     ier   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument aa or bb
!                         (then genbet = -1.0)
!
!     External functions called:
!        RAN1
!     Fortran  functions called:
!        EXP  LOG  MAX  MIN  SQRT
!
!-----------------------------------------------------------------------
!     From
!     RANLIB
!           Library of Fortran Routines for Random Number Generation
!                           Compiled and Written by:
!                                Barry W. Brown
!                                 James Lovato
!                    Department of Biomathematics, Box 237
!                    The University of Texas, M.D. Anderson Cancer Center
!                    1515 Holcombe Boulevard
!                    Houston, TX      77030
!
!**********************************************************************
!
!     REAL FUNCTION GENBET( A, B )
!               GeNerate BETa random deviate
!
!
!                              Function
!
!
!     Returns a single random deviate from the beta distribution with
!     parameters A and B.  The density of the beta is
!               x^(a-1) * (1-x)^(b-1) / B(a,b) for 0 < x < 1
!
!
!                              Arguments
!
!
!     A --> First parameter of the beta distribution
!                         REAL A
!
!     B --> Second parameter of the beta distribution
!                         REAL B
!
!
!                              Method
!
!
!     R. C. H. Cheng
!     Generating Beta Variate with Nonintegral Shape Parameters
!     Communications of the ACM, 21:317-322  (1978)
!     (Algorithms BB and BC)
!
!**********************************************************************

   implicit none

!     Function
!     --------
   real(kind=4) :: genbet

!     Arguments
!     ---------
   real(kind=4), intent(in) :: aa, bb
   integer, intent(inout) :: idum
   integer, intent(out) :: ier

!     Local declarations
!     ------------------
   real(kind=4), external :: ran1

   real(kind=4), parameter :: expmax=89.0
   !  Close to the largest number that can be exponentiated
   real(kind=4), parameter :: infnty=1.0e38
   !  Close to the largest representable single precision number

   real(kind=4) :: a, b, delta, r, s, t, u1, u2, v, w, y, z
   real(kind=4), save :: alpha, beta, gamma, k1, k2
   real(kind=4), save :: olda=-1.0, oldb=-1.0
   logical :: qsame

   !--------------------------------------------------------------------

   !  Test for valid input arguments
   if ( aa <= 0.0 .or. bb <= 0.0 ) then
      ier = 1
      genbet = -1.0
      return
   end if
   ier = 0
   qsame = (olda == aa) .and. (oldb == bb)
   olda = aa
   oldb = bb

   if ( min(aa,bb) > 1.0 ) then

      ! Algorithm BB

      ! Initialize
      if ( .not. qsame ) then
         a = min(aa,bb)
         b = max(aa,bb)
         alpha = a + b
         beta = sqrt((alpha-2.0)/ (2.0*a*b-alpha))
         gamma = a + 1.0/beta
      end if

      do
         u1 = ran1( idum )

         ! Step 1
         u2 = ran1( idum )
         v = beta*log(u1/ (1.0-u1))
         if ( v > expmax ) then
            w = infnty
         else
            w = a*exp(v)
         end if
         z = u1**2*u2
         r = gamma*v - 1.3862944
         s = a + r - w

         ! Step 2
         if ( s+2.609438 >= 5.0*z ) exit

         ! Step 3
         t = log(z)
         if ( s > t ) exit

         ! Step 4
         if ( (r+alpha*log(alpha/(b+w))) >= t ) exit
      end do

      ! Step 5
      if ( aa == a ) then
         genbet = w/(b+w)
      else
         genbet = b/(b+w)
      end if

   else

      ! Algorithm BC

      ! Initialize
      if ( .not. qsame ) then
         a = max(aa,bb)
         b = min(aa,bb)
         alpha = a + b
         beta = 1.0/b
         delta = 1.0 + a - b
         k1 = delta*(0.0138889+0.0416667*b)/(a*beta-0.777778)
         k2 = 0.25 + (0.5+0.25/delta)*b
      end if

      do
         u1 = ran1( idum )

         ! Step 1
         u2 = ran1( idum )
         if ( u1 < 0.5 ) then
            ! Step 2
            y = u1*u2
            z = u1*y
            if ( 0.25*u2+z-y >= k1 ) cycle
         else
            ! Step 3
            z = u1**2*u2
            if ( z <= 0.25 ) then
               v = beta*log(u1/ (1.0-u1))
               if ( v > expmax ) then
                  w = infnty
               else
                  w = a*exp(v)
               end if
               exit
            end if
            if ( z >= k2 ) cycle

         end if
         ! Step 4
         ! Step 5
         v = beta*log(u1/ (1.0-u1))
         if ( v > expmax ) then
            w = infnty
         else
            w = a*exp(v)
         end if
         if ( alpha*(log(alpha/(b+w))+v)-1.3862944 >= log(z) ) exit
      end do

      ! Step 6
      if ( a == aa ) then
         genbet = w/(b+w)
      else
         genbet = b/(b+w)
      end if

   end if

end function genbet
