function poissdev( av, idum, ier )

!-----------------------------------------------------------------------
!
!     Returns an integer random variable distributed according to a
!     Poisson distribution with parameter av.
!
!     av    - Input . Parameter of the Poisson distribution   - Real
!                     (0.0 < av <= 2.0e9)
!     idum  - Input . A negative integer to initialize the    - Integer
!             Output. random sequence. Thereafter do not
!                     alter this argument between successive
!                     deviates in a sequence
!     ier   - Output. Return code:                            - Integer
!                     0 = normal
!                     1 = invalid input argument av
!                         (then poissdev = -1)
!
!     External functions called:
!        EXPDV1  GAUSSBM1  RAN1
!     Fortran  functions called:
!        ABS  LOG  EXP  INT  MAX  MIN  REAL  SIGN  SQRT
!
!-----------------------------------------------------------------------
!
!     INTEGER FUNCTION IGNPOI( AV )
!
!                    GENerate POIsson random deviate
!
!
!                              Function
!
!
!     Generates a single random deviate from a Poisson
!     distribution with mean AV.
!
!
!                              Arguments
!
!
!     AV --> The mean of the Poisson distribution from which
!            a random deviate is to be generated.
!                              REAL AV
!
!     GENEXP <-- The random deviate.
!                              REAL GENEXP
!
!
!                              Method
!
!
!     Renames KPOIS from TOMS as slightly modified by BWB to use RANF
!     instead of SUNIF.
!
!     For details see:
!
!               Ahrens, J.H. and Dieter, U.
!               Computer Generation of Poisson Deviates
!               From Modified Normal Distributions.
!               ACM Trans. Math. Software, 8, 2
!               (June 1982),163-179
!
!**********************************************************************
!
!
!     P O I S S O N  DISTRIBUTION
!
!
!**********************************************************************
!
!     FOR DETAILS SEE:
!
!               AHRENS, J.H. AND DIETER, U.
!               COMPUTER GENERATION OF POISSON DEVIATES
!               FROM MODIFIED NORMAL DISTRIBUTIONS.
!               ACM TRANS. MATH. SOFTWARE, 8,2 (JUNE 1982), 163 - 179.
!
!     (SLIGHTLY MODIFIED VERSION OF THE PROGRAM IN THE ABOVE ARTICLE)
!
!**********************************************************************
!
!      INTEGER FUNCTION IGNPOI(IR,MU)
!
!     INPUT:  IR=CURRENT STATE OF BASIC RANDOM NUMBER GENERATOR
!             MU=MEAN MU OF THE POISSON DISTRIBUTION
!     OUTPUT: IGNPOI=SAMPLE FROM THE POISSON-(MU)-DISTRIBUTION
!
!     MUPREV=PREVIOUS MU, MUOLD=MU AT LAST EXECUTION OF STEP P OR B.
!     TABLES: COEFFICIENTS A0-A7 FOR STEP F. FACTORIALS FACT
!     COEFFICIENTS A(K) - FOR PX = FK*V*V*SUM(A(K)*V**K)-DEL
!
!-----------------------------------------------------------------------
!
!     March, 2003: changed to Fortran90 and
!     sexpo, snorm, ranf changed to expdv1, gaussbm1, ran1
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   integer :: poissdev

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: av
   integer, intent(inout) :: idum
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real, external :: ran1, expdv1, gaussbm1
   real, parameter :: a0=-.5,a1= .3333333,a2=-.2500068,a3= .2000118,a4=-.1661269, &
                      a5= .1421878,a6=-.1384794,a7= .1250060
   real :: b1, b2, c, c0, c1, c2, c3, d, del, difmuk, e
   real :: fk, fx, fy, g, mu, omega, p, p0, px, py, q, s, t, u, v, x, xx
   real :: muprev=0., muold=0.
   integer :: j, k, l, m
   logical :: kflag, lflag

   real :: fact(10)=(/ 1., 1., 2., 6., 24., 120., 720., 5040., 40320., 362880./)
   real :: pp(35)
   save

   !-----------------------------------------------------------------

   mu = av
   ier = 0
   if ( mu <= 0.0 .or. mu > 2.0e9 ) then
      ier = 1
      poissdev = -1
      return
   end if

   !  Separation of cases a and b

   if ( mu == muprev .or. mu >= 10.0 ) then

      !  Case  a. (recalculation of s, d, l if mu has changed)

      if ( mu /= muprev ) then
         muprev = mu
         s = sqrt(mu)
         d = 6.0*mu*mu
            !  The poisson probabilities pk exceed the discrete normal
            !  probabilities fk whenever k >= m(mu). l=int(mu-1.1484)
            !  is an upper bound to m(mu) for all mu >= 10 .
         l = int(mu-1.1484)
      end if

      !  Step n. Normal sample - gaussbm1(idum) for standard normal deviate

      g = mu + s*gaussbm1(idum)
      if ( g >= 0.0 ) then
         poissdev = int(g)

         !  Step i. Immediate acceptance if poissdev is large enough

         if ( poissdev >= l ) return

         !  Step s. Squeeze acceptance - ran1(idum) for (0,1)-sample u

         fk = real(poissdev)
         difmuk = mu - fk
         u = ran1(idum)
         if ( d*u >= difmuk*difmuk*difmuk ) return
      end if

      !  Step p. Preparations for steps q and h.
      !          (recalculations of parameters if necessary)
      !          .3989423=(2*pi)**(-.5)  .416667e-1=1./24.  .1428571=1./7.
      !          the quantities b1, b2, c3, c2, c1, c0 are for the hermite
      !          approximations to the discrete normal probabilities fk.
      !          c=.1069/mu guarantees majorization by the 'hat'-function.

      if ( mu /= muold ) then
         muold = mu
         omega = .3989423/s
         b1 = .4166667e-1/mu
         b2 = .3*b1*b1
         c3 = .1428571*b1*b2
         c2 = b2 - 15.*c3
         c1 = b1 - 6.*b2 + 45.*c3
         c0 = 1. - b1 + 3.*b2 - 15.*c3
         c = .1069/mu
      end if
      if ( g >= 0.0 ) then
         !  'subroutine' f is called (kflag=T for correct test)
         kflag = .true.
         lflag = .false.
      else
         lflag = .true.
      end if
      do
         if ( lflag ) then

            !  Step e. Exponential sample - expdv1(idum) for standard exponential
            !          deviate e and sample t from the laplace 'hat'
            !          (if t <= -.6744 then pk < fk for all mu >= 10.)

            e = expdv1(idum)
            u = ran1(idum)
            u = u + u - 1.0
            t = 1.8 + sign(e,u)
            if ( t <= (-.6744) ) cycle
            poissdev = int(mu+s*t)
            fk = real(poissdev)
            difmuk = mu - fk
            !  'subroutine' f is called (kflag=F for correct test)
            kflag = .false.
         else
            lflag = .true.
         end if

         !  Step f. 'subroutine' f. Calculation of px, py, fx, fy.

         if ( poissdev < 10 ) then
               !    Case poissdev < 10 uses factorials from table fact
            px = -mu
            py = mu**poissdev/fact(poissdev+1)
         else
               !    Case poissdev  >=  10 uses polynomial approximation
               !    a0-a7 for accuracy when advisable
               !    .8333333e-1=1./12.  .3989423=(2*pi)**(-.5)
            del = .8333333e-1/fk
            del = del - 4.8*del*del*del
            v = difmuk/fk
            if ( abs(v) <= 0.25 ) then
               px = fk*v*v* (((((((a7*v+a6)*v+a5)*v+a4)*v+a3)*v+a2)*v+a1)*v+a0) - del
            else
               px = fk*log(1.0+v) - difmuk - del
            end if
            py = .3989423/sqrt(fk)
         end if
         x = (0.5-difmuk)/s
         xx = x*x
         fx = -0.5*xx
         fy = omega* (((c3*xx+c2)*xx+c1)*xx+c0)
         if ( kflag ) then
            !  Step q. Quotient acceptance (rare case)

            if ( fy-u*fy <= py*exp(px-fx) ) return
         else
            !  Step h. Hat acceptance (e is repeated on rejection)

            if ( c*abs(u) <= py*exp(px+e)-fy*exp(fx+e) ) return
         end if
      end do

   else

      !  C a s e  b. (start new table and calculate p0 if necessary)

      muprev = 0.0
      if ( mu /= muold ) then
         muold = mu
         m = max(1, int(mu))
         l = 0
         p = exp(-mu)
         q = p
         p0 = p
      end if

      !  Step u. Uniform sample for inversion method

      do
         u = ran1(idum)
         poissdev = 0
         if ( u <= p0 ) return

         !  Step t. Table comparison until the end pp(l) of the
         !          pp-table of cumulative poisson probabilities
         !          (0.458=pp(9) for mu=10)

         if ( l /= 0 ) then
            j = 1
            if ( u > 0.458 ) j = min(l,m)
            do k = j, l
               if ( u <= pp(k) ) then
                  poissdev = k
                  return
               end if
            end do
            if ( l == 35 ) cycle
         end if

         !  Step c. Creation of new poisson probabilities p
         !          and their cumulatives q=pp(k)

         l = l + 1
         do k = l, 35
            p = p*mu/real(k)
            q = q + p
            pp(k) = q
            if ( u <= q ) then
               l = k
               poissdev = k
               return
            end if
         end do
         l = 35
      end do

   end if

end function poissdev
